import { Component, ViewChild } from '@angular/core';
import { App, NavParams, NavController, ViewController, FabContainer, Toast } from 'ionic-angular';
import { } from '@types/googlemaps';

import { AppData } from '../../models/app-data';
import { AppConfig } from '../../app/app-cfg';
import { Events } from 'ionic-angular';
import { BackendProvider } from '../../providers/backend/backend';
import { LoginPage } from '../../pages/login/login';
import { ProfilePage } from '../../pages/profile/profile';
import { GuiUtilComponent } from '../gui-util/gui-util';
import { AppInfoPage } from '../../pages/app-info/app-info';
import { UserProvider } from '../../providers/user/user';

/**
 * Generated class for the CmdMenu component.
 *
 * See https://angular.io/docs/ts/latest/api/core/index/ComponentMetadata-class.html
 * for more info on Angular Components.
 */
@Component({
  selector: 'cmd-menu',
  templateUrl: 'cmd-menu.html'
})
export class CmdMenu {

  // @ViewChild('uploadBtn') uploadBtnRef: ElementRef;
  @ViewChild(GuiUtilComponent) guiUtil: GuiUtilComponent;

  // cmdList: string[] = ["add", "camera", "alert", "list-box"];
  helpState: boolean = false;
  toast: Toast;
  // saveDisabled: boolean = false;
  commitBtnText = "Gefahrenstelle hinzufügen";
  loggedIn: boolean = false;
  
  constructor(public params: NavParams,
              private app: App,
              public appData: AppData,
              public viewCtrl: ViewController,
              private backend: BackendProvider,
              public navCtrl: NavController, 
              private user: UserProvider,
              public events: Events
            ) {
    console.log('Hello CmdMenu Component');

  }

  editSettings() {
  }

  commit() {
    // commit(fab?: FabContainer) {
    // if (fab) {
    //   fab.close();
    // }

    console.log('emit next nav...');
    this.events.publish(AppConfig.MSGEVT_NAV, { direction: AppConfig.NAV_NEXT });
  }

  goPrevious() {

    console.log('emit previous nav...');
    this.events.publish(AppConfig.MSGEVT_NAV, { direction: AppConfig.NAV_PREVIOUS });

  }

  toggleState() {
    this.helpState = !this.helpState;
  }

  saveError(err) {

    console.log(err);

    this.appData.helpText = "Beim Speichern ist ein Problem aufgetreten.";

    let errTxt = err.text();
    let jerr = JSON.parse(errTxt);

    switch (err.status) {
      case 0:
      this.appData.helpText += " Der Server scheint derzeit nicht erreichbar zu sein. Probieren Sie es später nochmals.";
      break;
      case 500:
        // TODO: server 500 problem mit model - send model to...??
        if (jerr.error.details) {
          for (let detail of jerr.error.details) {
            console.error(detail.message);
          }
        }
        break;
    }

    // this.events.publish(AppConfig.MSGEVT_HELP, {});
    this.guiUtil.showInfo(this.appData.helpText);

  }

  goHome() {
    this.navCtrl.push(AppInfoPage);
  }

  goProfile() {

    // console.log("store token");
    // var token = window.localStorage.setItem("token", "testtokenaösdfjsdfjsfj");
    // this.navCtrl.setRoot(LoginPage);
    // if(this.user.isAuthenticatedSync()) {
    //   this.navCtrl.push(ProfilePage);
    // } else {
    //   this.navCtrl.push(LoginPage);
    // }


    return this.user.isAuthenticated()
      .then( () => {
        console.log("user is logged in!");
        this.navCtrl.push(ProfilePage);
      })
      .catch( (err) => {
        console.log("not logged in!");
        this.navCtrl.setRoot(LoginPage);
      });

  }

  showHelp() {

    // this.events.publish(AppConfig.MSGEVT_HELP, {});
    this.guiUtil.showInfo(this.appData.helpText);
    
  }





}
