import { Component } from '@angular/core';
import { Platform } from 'ionic-angular/platform/platform';
import { AlertController } from 'ionic-angular/components/alert/alert-controller';
import { UserProvider } from '../../providers/user/user';
import { AppData } from '../../models/app-data';
import { ToastController } from 'ionic-angular/components/toast/toast-controller';

/**
 * Generated class for the GuiUtilComponent component.
 *
 * See https://angular.io/api/core/Component for more info on Angular
 * Components.
 */
@Component({
  selector: 'gui-util',
  templateUrl: 'gui-util.html'
})
export class GuiUtilComponent {

  constructor(public platform: Platform
              ,private alertCtrl: AlertController
              ,private appData: AppData
              ,private toastCtrl: ToastController
  ) {
    console.log('Hello GuiUtilComponent Component');
  }

  checkPlatform(msg: string) {
    
    if (!this.platform.is('cordova')) {
      
      console.warn('Cannot use physical camera because cordova is not available - Run in physical device!');
      let alert = this.alertCtrl.create({
          title: msg,
          subTitle: 'cordova device is needed.',
          buttons: ['OK']
        });
      alert.present();

      // add place holder picture instead
      // this.appData.issue.images.push("assets/img/image-placeholder.png");
      return false;
    }
    return true;
  }

  showDeviceId() {

    this.appData.getDeviceId()
    .then( (uuid: any) => {
      // todo: use it later for unauthenticated users
      console.log("device id: " + uuid);
      let alert = this.alertCtrl.create({
        title: "Device Id",
        subTitle: uuid,
        buttons: ['OK']
      });
      alert.present();
    })
    .catch((error) => {
      // todo: use it later for unauthenticated users
      console.info("device id error: " + error);
    });

    // add place holder picture instead
    // this.appData.issue.images.push("assets/img/image-placeholder.png");
    return false;
  }
 
  showInfo(msg: string) {
    
    let toast = this.toastCtrl.create({
      message: msg,
      duration: 5000,
      position: 'bottom',
      showCloseButton: true,
      closeButtonText: "Close"
    });
    
    toast.onDidDismiss(() => {
      // this.appData.issue.cmd.infoText = "";
      // console.log('Dismissed toast, reset info text.');
    });
  
    toast.present();    
  }
  

}
