import { Component } from '@angular/core';
import { AppData } from '../../models/app-data';
import { NavController, NavParams, Events } from 'ionic-angular';
import { BackendProvider } from '../../providers/backend/backend';
import { UserProvider } from '../../providers/user/user';
import { AppConfig } from '../../app/app-cfg';
import { IssueModel, IssueClass } from '../../models/issue-model';
import { UrlModel } from '../../models/url-model';
import { RequestModel } from '../../models/request-model';
import { TargetModel } from '../../models/target-model';
import { HazardOptionModel } from '../../models/hazard-option-model';

/**
 * Generated class for the ShowDetailComponent component.
 *
 * See https://angular.io/api/core/Component for more info on Angular
 * Components.
 */
@Component({
  selector: 'show-detail',
  templateUrl: 'show-detail.html'
})
export class ShowDetailComponent {

  urgencyLabels = { 0: "no", 1: "Niedrig", 2: "Mittel", 3: "Hoch"};
  issue: IssueClass;

  dangersRoot: TargetModel[];

  constructor(private appData: AppData,
    public navCtrl: NavController, 
    public navParams: NavParams,
    public events: Events,
    private backend: BackendProvider,
    private user: UserProvider) {
  
    console.log('Hello Show Detail Component');
    
    this.issue = IssueClass.getInstanceFrom(this.appData.issue)

    // alert('Show Detail Constructor');

    this.dangersRoot = this.issue.getSelectedHazardsRoot();
    
  }


  ionViewDidLoad() {
    console.log('ionViewDidLoad Show Detail Component');
    alert('Show Detail Load');
  }

  changeLocation() {
    alert("not implemented yet");
  }

  setIssue(issue: IssueModel) {
    this.issue = IssueClass.getInstanceFrom(issue);
  }

  get Urgency(): string {
    
    let urglevelIdx = this.issue.getUrgencyLevel();
    if(!urglevelIdx) {
      return "";
    }

    let urgLabel = this.urgencyLabels[this.issue.getUrgencyLevel()];
    if(!urgLabel) {
      return "";
    }
    
    return urgLabel;
  }

  get Description(): string {
    return this.issue.getDescription();
  }

  get Address(): string {
    return this.issue.getAddress();
  }


  get DangerSituations() {
    
    let dangers = this.issue.getSelectedHazards();
    if(!dangers) {
      dangers = "";
    }
    return dangers;
  }

  getDangerSub(rootDanger): HazardOptionModel[] {
    return this.issue.getSelectedHazardsSub(rootDanger);
  }
}
