import { Injectable } from '@angular/core';
import {AppModel} from './app-model';
import {IssueModel, IssueClass} from './issue-model';
// import { UniqueDeviceID } from '@ionic-native/unique-device-id';
import { Platform } from 'ionic-angular/platform/platform';
import { Http } from '@angular/http/src/http';
import { LocationModel } from './location-model';
import { CommandModel } from './command-model';
import { HazardOptionModel } from './hazard-option-model';
import { TargetModel } from './target-model';
import { retry } from 'rxjs/operator/retry';
import { UserModel } from './user-model';

@Injectable()
export class AppData implements AppModel {

  public static STATUS_NO_AUTH = 0;
  public static STATUS_AUTH = 1;
  public static STATUS_DRAFT = 2;

  public static MAP_HELP_TXT = [
    "GehFahrLos macht es möglich, Gefahren einfach zu melden."
    ,"Verschieben Sie den Marker oder klicken Sie diesen an, um die Adresse der Gefahrenstelle zu ändern."
  ];

  private static DEF_LOCATION =  { lat: 48.20230739999999, lng: 16.34502520000001, address: "1010, XGasse 9"} as LocationModel;

  public issue: IssueModel;
  cmd: CommandModel;
  
  public deviceid: any;
  private _helpText: string;
  status: number = 0;
  
  constructor( 
    private platform: Platform
    // ,private uniqueDeviceID: UniqueDeviceID
            ) {

    this.createModel();
    this.cmd = this.createCommandModel();
  }

  get helpText(): string {
    return this._helpText;
  }  
  set helpText(helpText: string) {
    this._helpText = helpText;
  }  

  createModel() {
    this.issue = IssueClass.createNew(AppData.DEF_LOCATION);
  }


  setLatLng(latitude: number, longitude: number) {
    this.issue.setLatLng(latitude, longitude);
  }


  createCommandModel() {
    return { 
      "commandList": [
        "camera","description","hazard","summary"], 
      "infoList": [
        "Fügen Sie der Gefahrenstelle ein Foto hinzu.",
        "Welche Gefahrensituationen sind zutreffend? Wähle eine Unterkategorie aus indem du das Menü aufklappst!",
        // "Klicke auf den Text, um die Unterkategorien anzuzeigen!",
        "Beschreiben Sie das Problem und die Dringlichkeit",
        "Sie sehen eine Übersicht. Sie können Ihre Angaben ändern, indem Sie zum entsprechenden Eingabeschritt navigieren. Klicken Sie hierzu auf die Symbole in der Navigationsleiste."],
      "commandIdx": 0,
      "command": "camera",
      "commandBtnIcon": "arrow-dropright-circle",
      saveDisabled: false
    };

  }
 

  getDeviceId(): Promise<any> {

    if(this.deviceid) {
      console.log("device id already assigned.");
      return Promise.resolve(this.deviceid);
    }

    console.log("trying to get device id...");
    if (!this.platform.is('cordova')) {
      console.log("no id because no cordova device found...");
      return new Promise((resolve) => { resolve("no device id on non cordova device"); });
    }

    return Promise.resolve("device id not available");

    // return this.uniqueDeviceID.get()
    //   .then((uuid: any) => {
    //     this.deviceid = uuid;
    //     console.log("got device id: " + uuid);
    //   })
    //   .catch((error: any) => { 
    //     console.info("couldn't get device id!");
    //     console.info(error); 
    //   });
  }
  


  testDraft() {

    this.cmd = this.createCommandModel();

    let draft: IssueModel = {
      // deviceid: "987654321", 
      issueid: 123456789,
      user_id: null,
      description: "testdraft", 
      importance: 0,
      images: [],
      location: { placeid: null, address: null, lat: 48.00230739999999, lng: 16.34502520000001},
      targetList: [
        { 
          participant: { label: "Fußgänger", value: false }, 
          hazardList: [ 
            { label: "enger Gehweg", value: false }, 
            { label: "schlecht geregelte Kreuzung", value: false },
            { label: "Schäden auf dem Gehweg / Stolpergefahr", value: false },
            { label:  "Fehlender Gehweg / Fußgängerübergang", value: false }
          ]
        },
        {
          participant: { label: "Öffentliche Verkehrsmittel", value: false }, 
          hazardList: [ 
            { label: "zu enger Bahnsteig / Haltestelle", value: false } 
            ,{ label: "Gefährlicher Zugang zu Haltestelle", value: false } 
            ,{ label: "fehlende Absicherung bei der Haltestelle", value: false } 
            ,{ label: "Vandalismus", value: false } 
          ]
        },
        {
          participant: { label: "Radfahrer", value: false }, 
          hazardList: [ 
            { label: "Schlaglöcher", value: false } 
            ,{ label: "enge Fahrspur", value: false } 
            ,{ label: "enger Radweg", value: false } 
            ,{ label: "Schienen / rutschige Fahrbahnbahnmarkierung", value: false } 
            ,{ label: "fehlende Abstellmöglichkeiten / Fahrradbügel", value: false } 
          ]
        },
        {
          participant: { label: "PKW", value: false }, 
          hazardList: [
            { label: "Hoher Randstein", value: false}
            ,{ label: "Unübersichtliche Kreuzung / Radübergang", value: false}
            ,{ label: "irreführende / mangelhafte Beschilderung", value: false}
          ]
        }
      ],
        getSelectedHazardsRoot():TargetModel[] { return new Array<TargetModel>();},
        getSelectedHazardsSub(target: TargetModel): HazardOptionModel[]{ return new Array<HazardOptionModel>();},
      
        setLatLng(latitude: number, longitude: number) {},
        getLatLng(): {lat: number, lng: number} { return {lat: 0, lng: 0}; },
        setAddress(address: string) {},
        setPlace(placeId: string, address: string) {},
        setHazardData() {}
      };


      this.issue = IssueClass.getInstanceFrom(draft);
  }
  
}