import {TargetModel} from './target-model';
import {CommandModel} from './command-model';
import {LocationModel} from './location-model';
import { HazardOptionModel } from './hazard-option-model';

export interface IssueModel {
  
  // deviceid: string;
  issueid: number;
  user_id: number;
  importance?: number;
  description?: string;
  images?: string[];
  location: LocationModel;
  targetList?: TargetModel[];

  getSelectedHazardsRoot():TargetModel[];
  getSelectedHazardsSub(target: TargetModel): HazardOptionModel[];
  setLatLng(latitude: number, longitude: number);
  getLatLng(): {lat: number, lng: number};
  setAddress(address: string);
  setPlace(placeId: string, address: string);
  setHazardData();

}

export class IssueClass implements IssueModel {

  // deviceid: string;
  issueid: number;
  user_id: number;
  importance?: number;
  description?: string;
  images?: string[];
  location: LocationModel;
  targetList?: TargetModel[];

  constructor() {
    this.issueid = null;
    this.location = {
        "address": null,
        "lat": null,
        "lng": null
    }
  }

  public setHazardData() {

    let tgList = 
    { "targetList": 
      [
        { "participant":{"categoryid": 161,"label":"Fu\u00dfg\u00e4nger","value": false},
        "hazardList":[
          {"categoryid": 162,"label":"enger Gehweg","value": false},
          {"categoryid": 163,"label":"fehlender Gehweg","value": false},
          {"categoryid": 164,"label":"Ampelanlage","value": false},
          {"categoryid": 165,"label":"Sch\u00e4den \/ Stolpergefahr","value": false},
          {"categoryid": 166,"label":"Kanaldeckel \/ Gitter","value": false},
          {"categoryid": 167,"label":"Beschilderung","value": false},
          {"categoryid": 168,"label":"Verunreinigung","value": false},
          {"categoryid": 169,"label":"Hindernisse","value": false}
        ]},
        
          {"participant":{"categoryid": 170,"label":"\u00d6ffentliche Verkehrsmittel","value": false},
          "hazardList":[
          {"categoryid": 171,"label":"Beschilderung","value": false},
          {"categoryid": 172,"label":"Gestaltung Haltestelle","value": false},
          {"categoryid": 173,"label":"Vandalismus","value": false},
          {"categoryid": 174,"label":"kein barrierefreier Einstieg","value": false}
          ]},
        
        {"participant":{"categoryid": 175,"label":"Radfahrer","value": false},
          "hazardList":[
          {"categoryid": 176,"label":"Stra\u00dfenzustand","value": false},
          {"categoryid": 177,"label":"enge Fahrspur","value": false},
          {"categoryid": 178,"label":"rutschige Oberfl\u00e4che","value": false},
          {"categoryid": 179,"label":"Abstellm\u00f6glichkeiten","value": false},
          {"categoryid": 180,"label":"L\u00fcckenschluss","value": false},
          {"categoryid": 181,"label":"Ampelschaltung","value": false},
          {"categoryid": 182,"label":"Beschilderung","value": false}]},
          
        {"participant":{"categoryid": 183,"label":"PKW","value": false},
          "hazardList":[
          {"categoryid": 184,"label":"Hoher Randstein","value": false},
          {"categoryid": 185,"label":"Un\u00fcbersichtliche Kreuzung","value": false},
          {"categoryid": 186,"label":"Beschilderung","value": false},
          {"categoryid": 187,"label":"Baustelle","value": false}
          ]}
        ]
      };

      this.targetList = tgList.targetList;
  }

  public getSelectedHazardsRoot():TargetModel[] {

    let tg: TargetModel[] = new Array<TargetModel>();
    if(!this.targetList) {
      
      return tg;
    }
    tg = <[TargetModel]>this.targetList.filter( (target) => target.participant.value);

    return tg;
    // selTargets.forEach( tgt => {
    //   let selHazards = tgt.hazardList.filter( (hazard) => hazard.value);
    //   if(selHazards.length>0) {
    //     hazards.concat(selHazards);
    //   }
    // });
  }
  
  public getSelectedHazardsSub(target: TargetModel): HazardOptionModel[] {
    
    return <HazardOptionModel[]>target.hazardList.filter( (hazard) => hazard.value);
  }
 
  getSelectedHazards() {

    let rootStr = "";
    let rootH = this.getSelectedHazardsRoot();
    rootH.forEach( tgt => {
      rootStr = rootStr + tgt.participant.label;
      let selSub = this.getSelectedHazardsSub(tgt);
      let subStr = selSub.join(", ");
      rootStr = rootStr + subStr;
    });
    return rootStr;
  }
  
  checkImageAndDescription(): string {
    if(this.images.length==0 || this.description.length==0) {
      return "Fügen Sie der Gefahrenstelle mindestens ein Foto oder eine Beschreibung hinzu.";
    }
    return null;
  }

  setPosition(position: Position) {
    
    console.log("set my position: " + position.coords.latitude + " / " + position.coords.longitude);
    this.location.lat = position.coords.latitude;
    this.location.lng = position.coords.longitude;
  }

  setLatLng(latitude: number, longitude: number) {
    this.location.lat = latitude;
    this.location.lng = longitude;
  }

  getLatLng(): {lat: number, lng: number} {
    return this.location;
  }

  setPlace(placeId: string, address: string) {
    this.location.placeid = placeId;
    this.setAddress(address);
  }

  setAddress(address: string) {
    this.location.address = address;
  }

  getAddress() {
    return this.location.address;
  }
 
  getUrgencyLevel() {
    return this.importance;
  }

  getDescription() {
    if(!this.description) {
      return "";
    }
    return this.description;
  }

  public static getInstanceFrom(issue: IssueModel): IssueClass {
    let issueObj: IssueClass = new IssueClass();
    // issueObj = <IssueClass>issue;

    // issueObj = IssueClass.cast(issue, IssueClass);

    issueObj.issueid = issue.issueid;
    issueObj.user_id = issue.user_id;
    issueObj.importance = issue.importance;
    issueObj.description = issue.description;
    issueObj.images = issue.images;
    issueObj.location = issue.location;
    issueObj.targetList = issue.targetList;

    return issueObj;
  }

  static cast<T>(obj: any, cl: { new(...args): T }): T {
    obj.__proto__ = cl.prototype;
    return obj;
  }

  public static createNew(location?: LocationModel) {
    
    console.log("create blank issue...");

    let blankobj: IssueModel = new IssueClass();

    if(location) {
      blankobj.location = location;
    } 

    return blankobj;
  }
  
}
