import { Component, ViewChild, ElementRef } from '@angular/core';
import { IonicPage, NavController, NavOptions, NavParams, Tabs } from 'ionic-angular';

import { TakePhotoComponent } from '../../components/take-photo/take-photo';
// import { SetPoiComponent } from '../../components/set-poi/set-poi';
import { DescriptionComponent } from '../../components/description/description';
import { HazardsComponent } from '../../components/hazards/hazards';
import { SummaryComponent } from '../../components/summary/summary';

import { AppConfig } from '../../app/app-cfg';
import { AppData } from '../../models/app-data';
import { Events } from 'ionic-angular';
import { CmdMenu } from '../../components/cmd-menu/cmd-menu';
import { ActionSheetController } from 'ionic-angular';

import { GuiUtilComponent } from '../../components/gui-util/gui-util';
import { SummaryPage } from '../summary/summary';
import { DescriptionPage } from '../description/description';
import { HazardsPage } from '../hazards/hazards';
import { PhotoPage } from '../photo/photo';
import { IssueModel } from '../../models/issue-model';

/**
 * Generated class for the IssueHandlerPage page.
 *
 * See http://ionicframework.com/docs/components/#navigation for more info
 * on Ionic pages and navigation.
 */
@IonicPage()
@Component({
  selector: 'page-issue-handler',
  templateUrl: 'issue-handler.html',
})
export class IssueHandlerPage {

  @ViewChild('myTabs') tabRef: Tabs;
  @ViewChild(GuiUtilComponent) guiUtil: GuiUtilComponent;

  photoTab: any = PhotoPage;
  descriptionTab: any = DescriptionPage;
  hazardTab: any = HazardsPage;
  summaryTab: any = SummaryPage;

  isActive = false;
  userForm: any;

  constructor(public navCtrl: NavController, 
              public navParams: NavParams,
              public appData: AppData,
              // public poi: SetPoiComponent,
              // private toastCtrl: ToastController,
              public events: Events,
              public actionSheetCtrl: ActionSheetController) {
    
    this.initCustom();
    this.subscribeEvents();
  }

  initCustom() {
    // let paramData = this.navParams.data;
    // if(paramData) {
    //   let issue: IssueModel = <IssueModel>paramData.data;
    //   this.showDetail.setIssue(issue);
    // }
    // this.appData.testDraft();
  }

  subscribeEvents() {
    // user navigation
    this.events.subscribe(AppConfig.MSGEVT_NAV, (event) => {
      // user and time are the same arguments passed in `events.publish(user, time)`
      console.log('tab nav: ', event);

      switch(event.direction) {
        case AppConfig.NAV_NEXT:
        this.goNext();
        break;
        case AppConfig.NAV_PREVIOUS:
        this.goPrevious();
        break;
        case AppConfig.NAV_SAVE:
        this.navCtrl.popToRoot();
        break;
      }
      // if(event.direction===AppConfig.NAV_NEXT) {
      //   this.goNext();
      // }
      // if(event.direction===AppConfig.NAV_PREVIOUS) {
      //   this.goPrevious();
      // }
    });

    // inform user about errors
    // this.events.subscribe(AppConfig.MSGEVT_HELP, (event) => {
    //   this.guiUtil.showInfo(this.appData.helpText);
    // });
    
    // change input data
    this.events.subscribe(AppConfig.MSGEVT_TABSELECT, (event) => {

      var idx = event.target;
      this.appData.cmd.commandIdx = idx;
      this.setInfoText();
  
      this.tabRef.select(idx);
    });
  }

  ionViewDidLoad() {
    console.log('ionViewDidLoad IssueHandlerPage');
  }

  tabClicked() {
    var idx = this.tabRef.getSelected().index;
    console.log("tab changed to idx: " + idx);
    this.appData.cmd.commandIdx = idx;
    this.setInfoText();
    this.guiUtil.showInfo(this.appData.helpText);
  }

  setInfoText() {
    var idx = this.tabRef.getSelected().index;
    console.log("set info text for index: " + idx);
    
    this.appData.helpText = this.appData.cmd.infoList[idx];
  }

  // refreshMap() {
  //   this.poi.refreshMap();
  // }


  goNext() {
    
    var idx = this.tabRef.getSelected().index;
    idx++;
    // console.log("next index: " + idx);
    this.tabRef.select(idx);
  }
    
  goPrevious() {
    // var prevTab = this.tabRef.previousTab(true);
    // this.tabRef.select(prevTab);
    var idx = this.tabRef.getSelected().index;
    // console.log("set info text: " + idx);
    idx--;
    // console.log("next index: " + idx);
    this.tabRef.select(idx);

  }   

  // showInfo() {
    
  //   let toast = this.toastCtrl.create({
  //     message: this.appData.helpText,
  //     duration: 5000,
  //     position: 'bottom',
  //     showCloseButton: true,
  //     closeButtonText: "Close",
  //     // cssClass: "toast-message"
  //   });
  
  //   toast.onDidDismiss(() => {
  //   });
  
  //   toast.present();    
  // }
      
  swipeEvent(e) {

    // DIRECTION_NONE         1
    // DIRECTION_LEFT         2
    // DIRECTION_RIGHT        4
    // DIRECTION_UP           8
    // DIRECTION_DOWN         16
    // DIRECTION_HORIZONTAL   6
    // DIRECTION_VERTICAL     24
    // DIRECTION_ALL          30

    // prevent swipe on maps page
    var idx = this.tabRef.getSelected().index;
    if(idx!==0) {
      console.log('emit swipte event: ' + e.direction);
      this.events.publish(AppConfig.MSGEVT_NAV, {direction: e.direction});
    }
    
    // if (e.direction == CmdMenu.NAV_NEXT) {
    //   //direction 2 = right to left swipe.
    //   console.log('emit next nav...');
    //   this.events.publish('tabnav', {direction: "next"});
    // }
    // if (e.direction == 4) {
    //   console.log('emit previous nav...');
    //   this.events.publish('tabnav', {direction: "prev"});
    // }
    
  }

  changeSettings() {
    console.log("change settings dialog...");
  }  

}
