import { Component } from '@angular/core';
import { IonicPage, NavController, NavParams, NavOptions } from 'ionic-angular';
import { FormBuilder, FormGroup, Validators  } from '@angular/forms';
import { RegisterPage } from '../register/register';
import { AppConfig } from '../../app/app-cfg';
import { MapPage } from '../map/map';
import { BackendProvider } from '../../providers/backend/backend';
import { UserProvider } from '../../providers/user/user';
import { FormControl } from '@angular/forms/src/model';
import { UrlModel } from '../../models/url-model';
import { TokenModel } from '../../models/token-model';
import { RequestModel } from '../../models/request-model';
import { RequestOptions } from '@angular/http';
import { UserModel } from '../../models/user-model';

/**
 * Generated class for the LoginPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@IonicPage()
@Component({
  selector: 'page-login',
  templateUrl: 'login.html',
})
export class LoginPage {

  credentialsForm: FormGroup;
  serverError: boolean = false;

  constructor(public navCtrl: NavController, 
              public navParams: NavParams,
              private backend: BackendProvider,
              private formBuilder: FormBuilder,
              private user: UserProvider) {
    console.log('LoginPage constructor');

    this.credentialsForm = this.formBuilder.group({
      email: [
        '', Validators.compose([
          Validators.email,
          Validators.required
        ])
      ],
      password: [
        '', Validators.compose([
          Validators.pattern(AppConfig.PWD_PATTERN),
          Validators.required
        ])
      ]
    });
  
  }

  ionViewDidLoad() {
    console.log('ionViewDidLoad LoginPage');
    this.serverError = false;
  }

  onSignIn() {
    console.log("TODO: login...");
    // this.logger.info('SignInPage: onSignIn()');
    if (this.credentialsForm.valid) {
      var email =  this.credentialsForm.controls['email'].value;
      var pwd = this.credentialsForm.controls['password'].value;
      this.user.login(email, pwd)
        .then((token: TokenModel) => {
          // console.log("login page: login success, request profile");
          // this.requestUserProfile(token);

          console.log("go to referrer page");
          this.serverError = false;
          this.navCtrl.pop();

        })
        .catch(() => {
          console.log("login page: login failed");
          this.serverError = true;
          this.credentialsForm.setErrors({'nologin': true});
          
          // const login = new FormControl("someLogin");
          // login.setErrors({
          //   "notUnique": true
          // });          
          // this.credentialsForm.controls['email'].setErrors({
          //   "emailAlreadyExist": true
          // });
        });
    } 
  }

  hasLoginError() {

    return this.credentialsForm.hasError('nologin') 
          || !this.credentialsForm.controls['email'].valid 
          || !this.credentialsForm.controls['password'].valid;
  }

  getCss() {
    if(this.serverError) {
      return "visible";
    }
    return "hidden";
  }

  onForgotPassword() {
    console.log("SignInPage: onForgotPassword()");
    // this.logger.info('SignInPage: onForgotPassword()');
  }

  onRegister() {
    console.log("go to register...");
    // this.navCtrl.push(RegisterPage);
    this.navCtrl.setRoot(RegisterPage);
  }  

  hasInvalidEmail() {
    return (!this.credentialsForm.controls.email.valid && this.credentialsForm.controls.email.dirty);
  }

  hasInvalidPassword() {
    return (!this.credentialsForm.controls.password.valid && this.credentialsForm.controls.password.dirty);
  }
}
