import { Injectable } from "@angular/core";
import { IssueModel, IssueClass } from "../../models/issue-model";
import { IssueMarkerModel, IssueMarkerClass } from "../../models/issue-marker-model";
import { UrlModel } from "../../models/url-model";
import { AppConfig } from "../../app/app-cfg";
import { RequestModel } from "../../models/request-model";
import { AppData } from "../../models/app-data";
import { BackendProvider } from "../../providers/backend/backend";
import { MapService } from "../../providers/map-service";
import { IssueHandlerPage } from "../issue-handler/issue-handler";
import { NavController, NavParams } from "ionic-angular";
import { LocationProvider } from "./location";
import * as MarkerClusterer from 'node-js-marker-clusterer';
import { TargetModel } from "../../models/target-model";
import { UrlProvider } from "../../providers/url/url";
import { Http } from "@angular/http";


@Injectable()
export class IssueManager {

  _viewClickCallback: (issue: IssueModel, marker: google.maps.Marker) => void;
  _editClickCallback: (issue: IssueModel, marker: google.maps.Marker) => void;
  _editDragEndCallback: (issue: IssueModel, marker: google.maps.Marker) => void;

  issuesMap: Map<number, IssueModel>;

  map: google.maps.Map;
  infowindow: google.maps.InfoWindow;
  markers: google.maps.Marker[];

  // issueMarkers: IssueMarkerModel[];
  // issues: IssueModel[];
  marker: google.maps.Marker;
  issue: IssueModel;

  
  constructor(
    public appData: AppData
    , private backend: BackendProvider
    , private mapService: MapService
    , private locationProvider: LocationProvider
    , public http: Http
  ) {
    this.issuesMap = new Map<number, IssueModel>();
  }


  setMap(map: google.maps.Map) {
    this.map = map;
  }

  public getIssueUrl(bounds: google.maps.LatLngBounds): string {

    // let boundsUrl = bounds.toUrlValue().replace(",","/");
    let ne = bounds.getNorthEast().toUrlValue();
    let sw = bounds.getSouthWest().toUrlValue();
    let boundsUrl = ne + "/" + sw;
    boundsUrl = boundsUrl.split(',').join('/');

    // let resUrl = "api/v1/gehfahrlos/issues/coordinates/48.20230739999999/16.34502520000001"

    let resUrl = "api/v1/gehfahrlos/issues/area/" + boundsUrl + "/load-issues";
    console.log("get issues from API for bounds: " + resUrl);

    return resUrl;
  }

  /**
   * Load all issues inside a specific area
   * returns issues not loaded previously (only new issues returned)
   * @param bounds google.maps.LatLngBounds area to load
   */
  public loadIssues(bounds: google.maps.LatLngBounds): Promise<any> {
    
    let resUrl = this.getIssueUrl(bounds);

    var urlCfg: UrlModel = { 
      // "path": "/geocode",
      "path": "/backendapi/",
      "proxyUrl": AppConfig.BACKEND_API_DOMAIN,
      "resource": resUrl
    }

    // let options: RequestOptionsArgs = null;
    let connectUrl = UrlProvider.getConnectUrl(urlCfg);

    return new Promise((resolve, reject) => { 
      this.http.get(connectUrl, null)
        .map(response => <IssueModel>response.json().data.full)
        // .catch(this.handleError)
        .subscribe((issuesNew) => {
          console.log("got data from server");
          this.setDiff(issuesNew, resolve);
        },
        (err) => {
          console.error("error loading issues: ");
          console.error(err);
        });
      
      let otherThingsHere = "others";
    });

  }

  private setDiff(newIssues: any, resolve) {

    console.log("retrieved: " + newIssues.length);
    var diff: IssueModel[] = new Array<IssueModel>();
    newIssues.forEach(element => {
      if(!this.issuesMap.has(element.issueid)) {
        this.issuesMap.set(element.issueid, element);
        diff.push(element);
      }
    });

    console.log("new issues selected: " + diff.length);
    // if(diff.length>0) {
    //   alert("new issues received: " + diff.length);
    // }
    resolve(diff);
  }

  canCreateAtLocation(): Promise<boolean> {

    console.info("can I create issue at:...");

    // sw?: LatLng|LatLngLiteral, ne?: LatLng|LatLngLiteral
    // let bounds: google.maps.LatLngBounds = new google.maps.LatLngBounds(this.appData.getLatLng(), this.appData.getLatLng());
    let posAt = this.appData.issue.getLatLng();
    let resUrl = "api/v1/gehfahrlos/issues/coordinates/" + posAt.lat + "/" + posAt.lng;

    var urlCfg: UrlModel = { 
      // "path": "/geocode",
      "path": "/backendapi/",
      "proxyUrl": AppConfig.BACKEND_API_DOMAIN,
      "resource": resUrl
    }
    console.log(resUrl);

    let reqParams: RequestModel = {urlCfg: urlCfg};

    return new Promise((resolve, reject) => { 
      this.backend.simpleGetRequest(reqParams)
      .then( (response) => {
        let res = response.json();
        if(res.data.full.length==0) {
          resolve(true);
        } else {
          resolve(false);
        }
      })
      .catch( (err) => {
        console.warn("server response error:");
        console.warn(err);
      });
    });
  }

  
  addDraft() {

    console.log("create new model.");
    this.appData.createModel();

    // set map center as current location
    var center = this.map.getCenter();
    this.appData.setLatLng(center.lat(), center.lng());
    
    this.locationProvider.setAddressFromPosition({"lat": center.lat(), "lng": center.lng()})
      .then( (address) => {
        // address already set
        this.appData.issue.setAddress(address);

        // var markerIcon = this.mapService.getIcon(MapService.MARKER_1);
        var markerIcon = this.mapService.getIcon(IssueMarkerClass.ICON_DRAFT);

        this.createMarkerEdit(center, address, markerIcon);
        this.showMarkerInfo(address);
        // this.marker.setVisible(true);
      })
      .catch( (err) => {console.warn(err)});

    // this.isNavOk = true;
  }  

  public createIssueRemote() {

    console.log("create issue on backend...");

    var urlConf: UrlModel = { 
      "path": "/backendapi/",
      "proxyUrl": AppConfig.BACKEND_API_DOMAIN,
      "resource": "api/v1/gehfahrlos/issues"
    }
    
    // this.appData.issue.location.address
    // this.backend.callAPI()

    let body: any = {
      issueid: null,
      location: this.appData.issue.location
    }

    // let bodyStr = JSON.stringify(this.appData.issue)
    let bodyStr = JSON.stringify(body);
    console.log("create remote issue: " + bodyStr);

    let reqParams: RequestModel = { urlCfg: urlConf, body: bodyStr };

    this.backend.authPostRequest(reqParams)
      .then((response)=> {
        var issue = <IssueModel>response.json();
        if(issue) {
          console.log("got issueid from api: " + issue.issueid);
          this.mountIssue(issue);
        }
      });
  }  


  public transformMarkerSaved() {

    this.marker.setDraggable(false);
    var icoName = this.selectIcon(this.appData.issue);
    var ico = this.mapService.getIcon(icoName);
    this.marker.setIcon(ico);
  }

  private mountIssue(issue: IssueModel) {

    // this.appData.issue.issueid = issue.issueid;
    var issueClass = IssueClass.getInstanceFrom(issue);
    this.appData.issue = issueClass;
    this.appData.issue.setHazardData();
    this.issuesMap.set(issue.issueid, this.appData.issue);
    var ico = this.mapService.getIcon(IssueMarkerClass.ICON_DRAFT);
    this.marker.setIcon(ico);
    var idx = this.markers.push(this.marker);
    // this.appData.createModel();
  }

  createMarkerEdit( location: google.maps.LatLng
                , title: string
                , markerIcon: google.maps.Icon) {


    console.log("try to add marker: " + title);
    this.marker = this.mapService.getMarker(location, markerIcon, title);

    this.marker.setMap(this.map);
    this.marker.setVisible(true);
    this.marker.setDraggable(true);

    // this.locationProvider.setAddressFromPosition(this.appData.getLatLng())
    //   .then( (address) => {
    //     // address already set
    //     this.showMarkerInfo(this.appData.issue.location.address);
    //   })
    //   .catch( (err) => {console.warn(err)});


    this.marker.addListener('dragend', (event) => {
      // this.location = event.latLng;
      console.log("dragend on map");

      var pos = this.marker.getPosition();
      this.appData.setLatLng(pos.lat(), pos.lng());
      
      this.locationProvider.setAddressFromPosition(this.appData.issue.getLatLng())
        .then( (address) => {
          // address already set
          this.showMarkerInfo(this.appData.issue.location.address);
          if(this._editDragEndCallback) {
            this._editDragEndCallback;
          }
        })
        .catch( (err) => {console.warn(err)});

      // this.showCreateMarkerDialog();
    });
  
    this.marker.addListener('click', (event) => {
      // this.location = event.latLng;
      console.log("clicked on marker");
      if(this._editClickCallback) {
        this._editClickCallback;
      }
  // this.navCtrl.push(IssueHandlerPage);
    });

  }

  public showMarkerInfo(winText: string) {

    if(!this.marker.getVisible()) {
      return;
    }

    if(this.infowindow) {
      this.infowindow.close();
    }
    this.infowindow = new google.maps.InfoWindow();
    this.infowindow.setContent(winText);
    this.infowindow.open(this.map, this.marker);

    // autosearch field
    // var srchInput = this.getDomSrchField();
    // srchInput.value = winText;
    // this.appData.issue.location.address = winText;

  }


  reloadIssues(bounds: google.maps.LatLngBounds) {
    
    console.log("TODO: clear old issues and display new ones.");

    var fugIcon: google.maps.Icon = {
      url: "assets/svg/marker_fug.svg",
      origin: new google.maps.Point(0, 0),
      size: new google.maps.Size(50, 50),
      scaledSize: new google.maps.Size(50, 50),
      anchor: new google.maps.Point(30, 0)
    }        
    
    // get new issues
    // let bounds = this.map.getBounds();

    this.loadIssues(bounds)
    .then((issues: Array<IssueModel>) => {

      this.markers = new Array<google.maps.Marker>();

      let issue: IssueModel;
      for(let issueData of issues) {

        issue = IssueClass.getInstanceFrom(issueData);

        var loc: google.maps.LatLng = new google.maps.LatLng(issue.location.lat, issue.location.lng);

        // var markerIcon = this.mapService.getIcon(MapService.MARKER_2);
        // var markerIcon = 'assets/svg/marker_fug.svg';
        let iconPath = this.selectIcon(issue);
        fugIcon.url = iconPath;

        var marker = this.mapService.getMarker(loc, fugIcon, null);
        this.assignMarker(marker, issue);        
      }

      var clusterImg = "https://raw.githubusercontent.com/googlemaps/v3-utility-library/master/markerclustererplus/images/m"
      // Add a marker clusterer to manage the markers.
      var markerCluster = new MarkerClusterer(this.map, this.markers,
        {imagePath: 'assets/cluster/spot'});
        // {imagePath: 'https://developers.google.com/maps/documentation/javascript/examples/markerclusterer/m'});
    })
    .catch((err) => {
      console.log(err.toString());
    });
  }

  assignMarker(curMarker: google.maps.Marker, issue: IssueModel) {
    
    curMarker.setMap(this.map);
    var idx = this.markers.push(curMarker);

    // curMarker.addListener('click', (issue) => {
    //   // this.location = event.latLng;
    //   let params: NavParams = new NavParams({data: issue});
    //   console.log("clicked on marker");
    //   this.navCtrl.push(DetailPage, params);
    // });


  }

  selectIcon(issue: IssueModel) {

    let selTargets: TargetModel[] = issue.getSelectedHazardsRoot();

    // let selTargets = issue.targetList.filter( (target) => target.participant.value);

    // TODO: eigene Gefahrenstellen
    // if(issue.user_id==this.user.userid) {
    //   // draft icon
    //   img = MapService.MARKER_1;
    //   return img;
    // }

    let catCnt = selTargets.length;
    let img: string;

    if(catCnt<1) {
      // draft icon
      img = IssueMarkerClass.ICON_DRAFT;
      return img;
    }

    if(catCnt>1) {
      // type multiple
      img = IssueMarkerClass.ICON_MULTIPLE;
      return img;
    }

    // return selTargets[0].participant.categoryid;
    let target = selTargets[0];
    // let subSelects: HazardOptionModel[] = issue.getSelectedHazardsSub(target);


    switch(target.participant.categoryid) {
      case 161:
      // Fußgänger
      img = IssueMarkerClass.ICON_FUSS;
      break;
      case 170:
      // Öffis
      img = IssueMarkerClass.ICON_OEFFI;
      break;
      case 175:
      // Radfahrer
      img = IssueMarkerClass.ICON_RAD;
      break;
      case 183:
      // PKW
      img = IssueMarkerClass.ICON_PKW;
      break;
    }

    return img;
  }
  

}