import { Injectable } from '@angular/core';
import { Http, RequestOptions, URLSearchParams } from '@angular/http';
import 'rxjs/add/operator/map';
import { Diagnostic } from '@ionic-native/diagnostic';
import { LocationAccuracy } from '@ionic-native/location-accuracy';
import { AlertController, Platform } from 'ionic-angular';
import { AppData } from '../../models/app-data';
import { UrlModel } from '../../models/url-model';
import { AppConfig } from '../../app/app-cfg';
import { RequestModel } from '../../models/request-model';
import { BackendProvider } from '../../providers/backend/backend';
import { LocationModel } from '../../models/location-model';

/*
  Generated class for the LocationProvider provider.

  See https://angular.io/guide/dependency-injection for more info on providers
  and Angular DI.
*/
@Injectable()
export class LocationProvider {

  _locationUpdateCallback: (position: Position) => void;
  
  constructor(public http: Http
    , private platform: Platform
    , private backend: BackendProvider
    , private alertCtrl: AlertController
    , private diagnostic: Diagnostic
    , private locationAccuracy: LocationAccuracy
    , public appData: AppData
  ) {
    console.log('Hello LocationProvider Provider');

  }

  public setLocationUpdateCallback(callbackMethod: (position: Position) => void) {
    this._locationUpdateCallback = callbackMethod;
  }

  checkLocationEnabled() {
    
    console.log("try to determine current postion...");

    console.warn('Cannot use physical camera because cordova is not available - Run in physical device!');
    
    // TODO: replace
    if(!this.checkPlatform("Diese Funktion muss auf einem cordova fähigen Gerät ausgeführt werden!")) {
      return;
    }

    this.diagnostic.isLocationEnabled()
      .then((isEnabled)=> {
        console.log("Is GPS enabled? " + isEnabled);
        if(isEnabled) {
          this.locateMe();
        } else {
          this.requestLocationEnable();
        }
      })
      .catch(this.handleLocationError);

  }

  requestLocationEnable() {
    this.locationAccuracy.canRequest()
      .then((canRequest: boolean) => {
        if(canRequest) {
          this.locationAccuracy.request(this.locationAccuracy.REQUEST_PRIORITY_HIGH_ACCURACY)
          .then(
            () => this.locateMe(),
            error => this.cannotGetPosition(error)
          );
        }
      });
  }

  // checkPlatform(msg: string) {
    
  //   if (!this.platform.is('cordova')) {
      
  //     console.warn('Cannot use physical camera because cordova is not available - Run in physical device!');
  //     let alert = this.alertCtrl.create({
  //         title: msg,
  //         subTitle: 'cordova device is needed.',
  //         buttons: ['OK']
  //       });
  //     alert.present();

  //     // add place holder picture instead
  //     // this.appData.issue.images.push("assets/img/image-placeholder.png");
  //     return false;
  //   }

  //   return true;
  // }
    
  handleLocationError(error) {


    console.error("error when processing location");
    console.error(error);
  }

  locateMe() {
  
    console.log("Trying to get my location...");
    // navigator.geolocation.watchPosition(
    //   (position) => this.updatePosition(position),
    //   () => this.handleLocationError(true)
    // );
    var options = {
      enableHighAccuracy: false,
      timeout: 20000,
      maximumAge: Infinity
    };

    navigator.geolocation.getCurrentPosition(
      position => { 
        console.log("found position!" + position.coords);

        let pos: LocationModel;
        pos.lat = position.coords.latitude;
        pos.lng = position.coords.longitude;

        this.setAddressFromPosition(pos)
        .then( (address) => {
          this.appData.issue.location.address = address;
          if(this._locationUpdateCallback) {
            this._locationUpdateCallback;
          }
        });
      },
      error => {
        console.log("error when trying to retrieve geolocation!");
        console.log(error.code);
        console.log(error.message);
      },
      options
    );
    
    
  }

  // updatePosition(position: Position) {
    
  //   this.appData.setPosition(position);
  //   this.setAddressFromPosition();
  //   var myLatLng = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);

  //   this.map.setCenter(myLatLng);
  //   this.map.panTo(myLatLng);
    
  //   // set marker position
  //   if(!this.marker) {
  //     this.createMarker(myLatLng, "Standort");
  //   } else {
  //     this.marker.setPosition(myLatLng);
  //   }

  //   this.marker.setAnimation(google.maps.Animation.DROP);
  //   // this.showInfoWindow("You are here!");
  //   // this.presentToast("Auf Karte klicken, Gefahrenstelle hinzufügen");
  // }

  public setAddressFromPosition(position: LocationModel): Promise<string> {

    // /latlng=40.714224,-73.961452&key=AIzaSyBXiTLQvfgdXo4GYyhjuAKSJI3ozNWwwBs
    // var position = this.appData.getLatLng();
    var posStr = position.lat + "," + position.lng;
    var urlStr = "?latlng=" + posStr + "&key=" + AppConfig.API_KEY;
    console.log("trying to get address from position: " + posStr);
    
    var url = "https://maps.googleapis.com/maps/api/geocode/json" + urlStr

    var urlCfg: UrlModel = { 
      // "path": "/geocode",
      "path": url,
      "proxyUrl": url,
      "resource": ""
    }

    let searchParam = new URLSearchParams();
    searchParam.append("latlng", posStr);
    searchParam.append("key", AppConfig.API_KEY);

    // alert("position: " + posStr);

    // let options: RequestOptionsArgs = {
    //   // headers: jwTokenInHeader
    //   // params: jwTokenParam
    //   search: searchParam
    // };

    let options: RequestOptions = new RequestOptions({ search: searchParam});

    // this.backend.callAPI(urlCfg, searchParam)
    let reqParams: RequestModel = {urlCfg: urlCfg, options};

    return new Promise( (resolve, reject) => {
      this.backend.simpleGetRequest(reqParams)
    // this.backend.simpleAPICall(urlCfg, searchParam)
      .then((response)=> {
        var address = response.json().results[0].formatted_address;
        var placeId = response.json().results[0].place_id;
        this.appData.issue.setPlace(placeId, address);
        console.log("service responded address: " + address);

        resolve(address);
        // this.changeDetector.detectChanges();
        // this.showMarkerInfo(address);
      })
      .catch( (err) => {
        console.log(err.toString());
        reject(err);
      });
    });

  }
  
  checkPlatform(msg: string) {
    
    if (!this.platform.is('cordova')) {
      
      console.warn('Cannot use physical camera because cordova is not available - Run in physical device!');
      let alert = this.alertCtrl.create({
          title: msg,
          subTitle: 'cordova device is needed.',
          buttons: ['OK']
        });
      alert.present();

      // add place holder picture instead
      // this.appData.issue.images.push("assets/img/image-placeholder.png");
      return false;
    }
    return true;
  }

  cannotGetPosition(error) {
    if(error) {
      console.log('User denied location accuracy - no position available', error);
    }
    let alert = this.alertCtrl.create({
      title: "Der Standortzugriff ist deaktiviert in den Systemeinstellungen!",
      subTitle: "Standort nicht verfügbar",
      buttons: ['OK']
    });
    alert.present();
  }
  
  
}
