import { Component,ViewChild } from '@angular/core';
import { Events, IonicPage, NavController, NavParams } from 'ionic-angular';
import { ToastController, Platform, AlertController, PopoverController } from 'ionic-angular';

import {} from '@types/googlemaps';
// import { AndroidPermissions } from '@ionic-native/android-permissions';
import { Diagnostic } from '@ionic-native/diagnostic';
import { LocationAccuracy } from '@ionic-native/location-accuracy';
import { URLSearchParams, RequestOptionsArgs, RequestOptions } from '@angular/http';

import { AppConfig } from '../../app/app-cfg';
import { AppData } from '../../models/app-data';
import { UrlModel } from '../../models/url-model';
import { LoginPage } from '../../pages/login/login';
import { IssueHandlerPage } from '../../pages/issue-handler/issue-handler';
import { MapService } from '../../providers/map-service';
import { BackendProvider } from '../../providers/backend/backend';
import { retry } from '@ionic/db/node_modules/rxjs/operator/retry';
import { UserProvider } from '../../providers/user/user';
import { CmdMenu } from '../../components/cmd-menu/cmd-menu';
import { IssueModel, IssueClass } from '../../models/issue-model';

import { LocationModel } from '../../models/location-model';
import { GuiUtilComponent } from '../../components/gui-util/gui-util';
import { ProfilePage } from '../profile/profile';
import { Subject, BehaviorSubject } from 'rxjs';
import { TokenModel } from '../../models/token-model';
import { RequestModel } from '../../models/request-model';
import { SummaryPage } from '../summary/summary';
import { DetailPage } from '../detail/detail';
import { LocationProvider } from './location';
import { TargetModel } from '../../models/target-model';
import { HazardOptionModel } from '../../models/hazard-option-model';
import { IssueMarkerClass, IssueMarkerModel } from '../../models/issue-marker-model';
import { IssueManager } from './issue-manager';

declare var google;

/**
 * Generated class for the MapPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@IonicPage()
@Component({
  selector: 'page-map',
  templateUrl: 'map.html',
})
export class MapPage {

  @ViewChild('addressfield') input; 
  @ViewChild(GuiUtilComponent) guiUtil: GuiUtilComponent;

  map: google.maps.Map;
  // marker: google.maps.Marker;
  // infowindow: google.maps.InfoWindow;

  boundsChanged : boolean = false;
  centerChanged : boolean = false;

  isAuth$: BehaviorSubject<TokenModel>;
  // navStep: number = 0;


  constructor(public navCtrl: NavController, 
              public navParams: NavParams,
              private locationProvider: LocationProvider,
              // private androidPermissions: AndroidPermissions,
              public mapService: MapService,
              public platform: Platform,
              // public toastCtrl: ToastController,
              private backend: BackendProvider,
              private user: UserProvider,
              private alertCtrl: AlertController,
              private diagnostic: Diagnostic,
              private locationAccuracy: LocationAccuracy,
              public events: Events,
              public popoverCtrl: PopoverController,
              public appData: AppData
              ,private issueManager: IssueManager
            ) {

    this.subscribeEvents();
    // this.navStep = 0;
    this.appData.status = AppData.STATUS_NO_AUTH;

  }

  ionViewCanEnter() {
  }

  ionViewDidLoad() {
    
    console.log('ionViewDidLoad MapPage');
    this.appData.helpText = AppData.MAP_HELP_TXT[0];
    this.guiUtil.showInfo(this.appData.helpText);

    this.user.isAuthenticated().then((token) => {
      this.appData.status = AppData.STATUS_AUTH;
      // this.navStep = 1;
    });
  }

  ngAfterViewInit() {
    this.locationProvider.checkLocationEnabled();
    try {
      this.initMap();
    } catch(error) {
      alert(error.toString());
    }
  }

  // ionViewWillEnter() {
  //   var currCenter = this.map.getCenter();
  //   google.maps.event.trigger(this.map, 'resize');
  //   this.map.setCenter(currCenter);
  // }
  
  subscribeEvents() {
    // inform user about errors
    // this.events.subscribe(AppConfig.MSGEVT_MARKER, (event) => {
    //   if(this.marker) {
    //     this.marker.setVisible(true);
    //     this.setAddressFromPosition();
    //   }
    // });

    // this.events.subscribe(AppConfig.MSGEVT_HELP, (event) => {
    //   this.guiUtil.showInfo(this.appData.helpText);
    // });
    this.isAuth$ = this.user.getAuthSubject();
    this.isAuth$.asObservable().subscribe((token) => {
      // console.log("authentication change!" + (token===null));
      let condition: boolean = (token!==null && token.access_token!==null);
      console.log("authentication change!" + condition);

      if(condition) {
        this.appData.status = AppData.STATUS_AUTH;
        // this.navStep = 1;
      }
      // alert("logged in? " + loggedIn);
    });

    this.locationProvider.setLocationUpdateCallback(this.updatePosition);
  }

  goLogin() {
    this.navCtrl.push(LoginPage);    
  }


  updatePosition(position: Position) {
    
    // this.appData.setPosition(position);
    // this.setAddressFromPosition();
    var myLatLng = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
  
    this.map.setCenter(myLatLng);
    this.map.panTo(myLatLng);
    
    // set marker position
    // if(!this.marker) {
    //   this.issueManager.createMarker(myLatLng, "Standort");
    // } else {
    //   this.marker.setPosition(myLatLng);
    // }
  
    // this.marker.setAnimation(google.maps.Animation.DROP);
    // this.showInfoWindow("You are here!");
    // this.presentToast("Auf Karte klicken, Gefahrenstelle hinzufügen");
  }
  

  initMap() {
    
    console.log('init map, set default position...');

    var mapPosition = this.appData.issue.getLatLng();
    var defPos = new google.maps.LatLng(this.appData.issue.getLatLng());

    var mapProp = {
        center: mapPosition,
        zoom: 15,
        mapTypeControl: false,
        streetViewControl: false,
        mapTypeId: google.maps.MapTypeId.ROADMAP,
        // draggableCursor: 'crosshair',    
        zoomControlOptions: {
          position: google.maps.ControlPosition.TOP_LEFT
        }
    };

    var mapDOMEl = document.getElementById("map");
    this.map = new google.maps.Map(mapDOMEl, mapProp);    
    
    // TODO: set it on map loaded
    this.issueManager.setMap(this.map);

    google.maps.event.addListener(this.map, 'click', (event) => {
      // this.location = event.latLng;
      console.log("clicked on map");
      // this.showCreateMarkerDialog();
    });

    google.maps.event.addListener(this.map, "idle", () => {
      
      console.log("check bounds change: " + this.boundsChanged);
      if(this.boundsChanged) {
        console.log("request for issues of new bounds...");
        this.reloadIssues();
        this.boundsChanged = false;
      }
      if(this.centerChanged) {
        console.log("center changed: " + this.map.getCenter().toString());
      }
    });

    this.map.addListener("bounds_changed", () => {
      this.boundsChanged = true;
    });
  
    this.map.addListener("center_changed", () => {
      this.centerChanged = true;
    });
  
    //Associate the styled map with the MapTypeId and set it to display.
    var styledMapType = this.mapService.getMapStyleDefs(null);
    this.map.mapTypes.set('styled_map', styledMapType);

    // traffic layer
    // var trafficLayer = new google.maps.TrafficLayer();    
    // public transport
    // var transitLayer = new google.maps.TransitLayer();
    // bicycle layer
    var bikeLayer = new google.maps.BicyclingLayer();
    bikeLayer.setMap(this.map);

    // this.map.data.setStyle({
    //   icon: "./assets/svg/ic_edit_location_black_24px.svg",
    //   fillColor: 'green'
    // });

    // this.map.data.setStyle((feature) => {
    //   return /** @type {google.maps.Data.StyleOptions} */({
    //    cursor : 'crosshair',
    //    visible: false
    //  });
    // });

    this.map.setMapTypeId('styled_map');
  }

  reloadIssues() {
    
    console.log("reload issues...");

    // get new issues
    let bounds = this.map.getBounds();
    this.issueManager.reloadIssues(bounds);
  }


  addDraft() {

    console.log("add draft on map.");

    this.appData.status = AppData.STATUS_DRAFT;
    // this.navStep = 2;
    this.issueManager.addDraft();
  }  


  createIssue() {

    this.issueManager.canCreateAtLocation()
    .then( (canCreate) => {
      if(canCreate) {
        this.issueManager.createIssueRemote();
        this.navCtrl.push(IssueHandlerPage);
      } else {
        let alert = this.alertCtrl.create({
          title: "Gefahrenstelle vorhanden",
          subTitle: "Die Gefahrenstelle an dieser Position wurde schon gemeldet!",
          buttons: ['OK']
        });
        alert.present();
      
      }

    })
    .catch( (err) => {
      console.error(err);
      // alert("Gefahrenstelle bereits vorhanden!");
    });
  }



  public refreshMap() {

    // this.backend.testThis();

    // console.log("refresh map...");
    google.maps.event.trigger(this.map, 'resize');
  }


// #region auto search

  // getDomSrchField() {

  //   console.log("get dom field...");
  //   var srchInput = this.input.getNativeElement();
  //   var srchInputNative = srchInput.getElementsByTagName('input');
  //   if(srchInputNative && srchInputNative.length>0) {
  //     console.log("found srchInput...");
  //     return srchInputNative[0];
  //   }

  //   console.log("couldn't get srchInput, try another way...");
  //   var nativeHomeInputBox = document.getElementById('addressfield');
  //   var nat = nativeHomeInputBox.getElementsByTagName('input')[0];
  //   return nat;
  //   // var srchInput = this.searchElementRef.nativeElement;
  // }

  // initAutoSearch() {

  //   console.log("prepare address search...");
  //   var srchInput = this.getDomSrchField();
  //   // display addressfield inside map
  //   // this.map.controls[google.maps.ControlPosition.TOP_LEFT].push(srchInput);
  //   // this.map.controls[google.maps.ControlPosition.TOP_LEFT].push(types);    

  //   // var autocompleteOptions: google.maps.places.AutocompleteOptions = {componentRestrictions: {country: 'at'}};
  //   // var autocomplete = new google.maps.places.Autocomplete(srchInput, autocompleteOptions);
  //   var autocomplete = new google.maps.places.Autocomplete(srchInput);
    
  //   autocomplete.bindTo('bounds', this.map);

  //   this.infowindow = new google.maps.InfoWindow();
  //   // var marker = new google.maps.Marker(
  //   //   // {
  //   //   //   map: this.map,
  //   //   //   anchorPoint: new google.maps.Point(0, -29)
  //   //   // }
  //   // );

  //   autocomplete.addListener('place_changed', () => {

  //     console.log("place changed...");
  //     this.infowindow.close();
  //     this.marker.setVisible(false);
  //     var place = autocomplete.getPlace();
  //     if (!place.geometry) {
  //       // User entered the name of a Place that was not suggested and
  //       // pressed the Enter key, or the Place Details request failed.
  //       window.alert("No details available for input: '" + place.name + "'");
  //       return;
  //     }

  //     // If the place has a geometry, then present it on a map.
  //     if (place.geometry.viewport) {
  //       this.map.fitBounds(place.geometry.viewport);
  //     } else {
  //       this.map.setCenter(place.geometry.location);
  //       this.map.setZoom(17);  // Why 17? Because it looks good.
  //     }
  //     // this.marker.setIcon(/** @type {google.maps.Icon} */({
  //     //   url: place.icon,
  //     //   size: new google.maps.Size(71, 71),
  //     //   origin: new google.maps.Point(0, 0),
  //     //   anchor: new google.maps.Point(17, 34),
  //     //   scaledSize: new google.maps.Size(35, 35)
  //     // }));
  //     this.marker.setPosition(place.geometry.location);
  //     this.marker.setVisible(true);

  //     var address = '';
  //     if (place.address_components) {
  //       address = [
  //         (place.address_components[0] && place.address_components[0].short_name || ''),
  //         (place.address_components[1] && place.address_components[1].short_name || ''),
  //         (place.address_components[2] && place.address_components[2].short_name || '')
  //       ].join(' ');
  //     }

  //     this.appData.issue.location.address = address;
  //     // this.changeDetector.detectChanges();
  //     console.log("autocomplete: detect changes1");

  //     this.infowindow.setContent('<div><strong>' + place.name + '</strong><br>' + address);
  //     this.infowindow.open(this.map, this.marker);
  //   });    
    
  // }

// #endregion autosearch

  test() {

    // alert(bounds.toString());
    // this.backend.testThis();
    // this.navCtrl.push(ProfilePage);
    // this.marker.setIcon(IssueMarkerClass.ICON_FUSS);

    this.refreshMap();
  }

  
  
}
