import { ChangeDetectorRef, Component, ViewChild } from '@angular/core';
import { IonicPage, NavController, 
          NavParams, 
          Nav , 
          Tabs, 
          AlertController,
          Platform 
} from 'ionic-angular';
import { Headers } from '@angular/http';

// import { resolveDefinition } from '@angular/core/src/view/util';
import { DomSanitizer } from '@angular/platform-browser';
import { AppData } from '../../models/app-data';
import { Camera, CameraOptions } from '@ionic-native/camera';
// import { Camera, CameraOptions } from 'ionic-native';
import { FileChooser } from '@ionic-native/file-chooser';
import { File, FileEntry, FileError, MetadataCallback, Metadata } from '@ionic-native/file';
import { FilePath } from '@ionic-native/file-path';
import { FileTransfer, FileUploadOptions, FileTransferObject } from '@ionic-native/file-transfer';
import { ImageResizer, ImageResizerOptions } from '@ionic-native/image-resizer';
import { ImagePicker } from '@ionic-native/image-picker';
import { Message } from '@angular/compiler/src/i18n/i18n_ast';

import { GuiUtilComponent } from '../../components/gui-util/gui-util';
import { ShowPhotoComponent } from '../../components/show-photo/show-photo';
import { AppConfig } from '../../app/app-cfg';
import { UrlModel } from '../../models/url-model';
import { UrlProvider } from '../../providers/url/url';
import { RequestModel } from '../../models/request-model';
import { BackendProvider } from '../../providers/backend/backend';


/**
 * Generated class for the PhotoPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@IonicPage()
@Component({
  selector: 'page-photo',
  templateUrl: 'photo.html',
})
export class PhotoPage {

  @ViewChild(ShowPhotoComponent) showPhotoComp: ShowPhotoComponent;
  @ViewChild(GuiUtilComponent) guiUtil: GuiUtilComponent;

  tab: Tabs;
  // maxReached: boolean = false;

  doUpload: boolean = false;
  imageURI: any;
  imageFileName: any;

  imageSize: any = {
    quality: 70,
    width: 400,
    height: 400,
  };

  constructor(private changeDetector: ChangeDetectorRef,
    public navCtrl: NavController,
    public navParams: NavParams,
    private nav: Nav,
    private backend: BackendProvider,
    private appData: AppData,
    public platform: Platform,
    private alertCtrl: AlertController,
    private fileChooser: FileChooser,
    private file: File,
    private transfer: FileTransfer,
    private sanitizer: DomSanitizer,
    private filePath: FilePath,
    // public loadingCtrl: LoadingController,
    // private camera: Camera,
    private imageResizer: ImageResizer,
    private imagePicker: ImagePicker,
    private camera: Camera
  ) {

    console.log('Hello TakePhoto Page: ' + this.nav.id);
    this.tab = this.navCtrl.parent;
  }

  ionViewDidLoad() {
  }

  //#region InputValidation

  goNext() {

    // var msg = this.appData.checkImageAndDescription();
    // if(msg) {
    //   alert(msg);
    // }
    // var idx = this.tab.parent.getActive().index;
    // var idx = this.tab.parent.getActive().index;
    // var idx1 = this.tab.getSelected().index;
    // console.log("current tab index: " + idx);
    // console.log("current tab index1: " + idx1);

    // this.tab.select(2);
    this.tab.select(1);
  }

  // checkMaxReached(): boolean {
  //   this.maxReached = false;
  //   if(this.appData.issue.images.length>=TakePhotoComponent.MAX_SIZE_PHOTO) {
  //     this.maxReached = true;
  //   }

  //   return this.maxReached;
  // }

  //#endregion


  takePhoto() {
    console.log("trying to take photo...");

    if (!this.guiUtil.checkPlatform("Diese Funktion muss auf einem cordova fähigen Gerät ausgeführt werden!")) {
      return;
    }

    if (this.showPhotoComp.checkMaxReached()) {
      console.log("reached maximum picture count! No more possible.");
      return;
    }

    const options: any = {
      quality: this.imageSize.quality,
      destinationType: this.camera.DestinationType.FILE_URI,
      encodingType: this.camera.EncodingType.PNG,
      mediaType: this.camera.MediaType.PICTURE,
      targetWidth: this.imageSize.width,
      targetHeight: this.imageSize.height
      // destinationType: this.camera.DestinationType.FILE_URI,
      // sourceType: Camera.PictureSourceType.PHOTOLIBRARY
    };

    this.camera.getPicture(options)
      .then((imageuri) => { this.convertPath(imageuri); })
      .catch((error) => {
        console.error("error taking a picture.");
        console.error(error);
      });
  }

  selectPhoto() {

    console.log("select existing photo...");

    if (!this.guiUtil.checkPlatform("Diese Funktion muss auf einem cordova fähigen Gerät ausgeführt werden!")) {
      return;
    }

    if (this.showPhotoComp.checkMaxReached()) {
      console.log("reached maximum picture count! No more possible.");
      return;
    }

    this.fileChooser.open()
      .then((uri) => {
        // this.resizeImage(uri);
        this.convertPath2(uri);
      })
      .catch(e => this.onErrorPhotoSelect("selectPhoto", e));
  }

  pickImage() {

    console.log("pick images...");

    if (this.showPhotoComp.checkMaxReached()) {
      console.log("reached maximum picture count! No more possible.");
      return;
    }

    var imgCntLef: number = ShowPhotoComponent.MAX_SIZE_PHOTO - this.appData.issue.images.length;
    console.log("can add more images: " + imgCntLef);

    var options = {
      // max images to be selected, defaults to 15. If this is set to 1, upon
      // selection of a single image, the plugin will return it.
      maximumImagesCount: imgCntLef,

      // max width and height to allow the images to be.  Will keep aspect
      // ratio no matter what.  So if both are 800, the returned image
      // will be at most 800 pixels wide and 800 pixels tall.  If the width is
      // 800 and height 0 the image will be 800 pixels wide if the source
      // is at least that wide.
      width: 200,
      height: 200,

      // quality of resized image, defaults to 100
      quality: 75
    };

    this.imagePicker.getPictures(options)
      .then((results) => {
        for (var i = 0; i < results.length; i++) {
          console.log("selected image: " + results[i]);
          this.convertPath(results[i]);
        }
      }, (err) => { console.error(err); });
  }


  resizeImage(imgUri: any) {

    let options = {
      uri: imgUri,
      folderName: this.file.dataDirectory,
      quality: 70,
      width: this.imageSize.width,
      height: this.imageSize.height
    } as ImageResizerOptions;

    // let options = {
    //   uri: resolvedPath.nativeURL,
    //   // folderName: 'Protonet',
    //   quality: this.imageSize.quality,
    //   width: this.imageSize.width,
    //   height: this.imageSize.height
    //  } as ImageResizerOptions;

    this.imageResizer
      .resize(options)
      .then((filePath: string) => {
        console.log("resized image: ", filePath);
        // this.convertPath(filePath);
        //  resolvedPath.file((file: Blob) => { this.readFileContent(file, resolvedPath);}, this.onFileError)

        this.file.resolveLocalFilesystemUrl(filePath)
          .then((resolvedPath: FileEntry) => {
            resolvedPath.file((file: Blob) => { this.readFileContent(file, resolvedPath); }, this.onFileError)
            // this.checkLoad(resolvedPath, checkSize);
          })
          .catch((error) => { this.onErrorPhotoSelect("resolveLocalFilesystemUrl", error); })

      })
      .catch(e => console.error(e));
  }

  convertPath(nativePath: any) {

    console.log("datadir: " + this.file.dataDirectory);
    console.log("nativePath: " + nativePath);

    this.filePath.resolveNativePath(nativePath)
      .then((localPath) => {
        console.log("localPath: " + localPath);

        this.file.resolveLocalFilesystemUrl(localPath)
          .then((resolvedPath: FileEntry) => {
            resolvedPath.file((file: Blob) => { this.readFileContent(file, resolvedPath); }, this.onFileError)
            // this.checkLoad(resolvedPath, checkSize);
          })
          .catch((error) => { this.onErrorPhotoSelect("resolveLocalFilesystemUrl", error); })
      })
      .catch((err) => { this.onErrorPhotoSelect("resolveNativePath", err); });
  }


  convertPath2(nativePath: any) {

    console.log("datadir: " + this.file.dataDirectory);
    console.log("nativePath: " + nativePath);

    this.filePath.resolveNativePath(nativePath)
      .then((localPath) => {
        console.log("localPath: " + localPath);

        this.resizeImage(localPath);

      })
      .catch((err) => { this.onErrorPhotoSelect("resolveNativePath", err); });
  }


  readFileContent(file: Blob, pathToImage: FileEntry) {

    console.log("trying to read file content...");
    var reader = new FileReader();

    reader.onload = (ev: Event) => {
      console.log("image onload");
    }
    reader.onabort = (ev: Event) => {
      console.log("image load aborted");
    }
    reader.onprogress = (ev: ProgressEvent) => {
      console.log("image load aborted");
    }

    reader.onloadend = (readerEvt: any) => {
      // reader.onload = (readerEvt: any) => {
      console.log("image loaded!");
      var src = readerEvt.target.result;
      console.log(src);
      // src = src.split("base64,");
      this.appData.issue.images.push(src);
      this.changeDetector.detectChanges();
      // var contentAsBase64EncodedString = src[1];

      this.uploadFile(pathToImage);
    };

    reader.onerror = (ev: ErrorEvent) => {
      console.log("error loading image...");
      console.log(ev.message);
      console.log(ev.error);
    }

    reader.readAsDataURL(file);
    // reader.readAsBinaryString(file);

  }


  // checkPlatform(msg: string) {

  //   if (!this.platform.is('cordova')) {

  //     console.warn('Cannot use physical camera because cordova is not available - Run in physical device!');
  //     let alert = this.alertCtrl.create({
  //         title: msg,
  //         subTitle: 'cordova device is needed.',
  //         buttons: ['OK']
  //       });
  //     alert.present();

  //     // add place holder picture instead
  //     // this.appData.issue.images.push("assets/img/image-placeholder.png");
  //     return false;
  //   }

  //   return true;
  // }

  getUrl(url) {
    return this.sanitizer.bypassSecurityTrustUrl(url);
  }

  onErrorPhotoSelect(func: string, e) {
    console.error("error at function: " + func);
    console.error(e);
  }

  onFileError(err: FileError) {
    this.onErrorPhotoSelect("onFileError", err);
  }

  toDataURL(url, callback) {
    var xhr = new XMLHttpRequest();
    xhr.onload = function () {
      var reader = new FileReader();
      reader.onloadend = function () {
        callback(reader.result);
      }
      reader.readAsDataURL(xhr.response);
    };
    xhr.open('GET', url);
    xhr.responseType = 'blob';
    xhr.send();
  }


  //#region TestCode

  getImage() {
    const options: CameraOptions = {
      quality: 100,
      destinationType: this.camera.DestinationType.FILE_URI,
      sourceType: this.camera.PictureSourceType.PHOTOLIBRARY
    }

    this.camera.getPicture(options).then((imageData) => {
      this.imageURI = imageData;
    }, (err) => {
      console.log(err);
      this.guiUtil.showInfo(err.message);
    });
  }



  uploadFile(pathToImage: FileEntry) {
    // if (this.doUpload === false) {
    //   console.log("image upload disabled.");
    //   return;
    // }


    var urlCfg: UrlModel = { 
      "path": "/backendapi/",
      "proxyUrl": AppConfig.BACKEND_API_DOMAIN,
      "resource": "api/v1/gehfahrlos/images"
    }

    let reqParams: RequestModel = {urlCfg: urlCfg };
    this.backend.getAuthRequestHeader(reqParams) 
      .then( (reqParams) => {
        this.sendPhoto(pathToImage, reqParams);
    });

    
  }

  private sendPhoto(pathToImage: FileEntry, reqParams: RequestModel) {


    var imgToUpload = pathToImage.nativeURL;
    console.log("trying to upload image: " + imgToUpload);

    // console.log("fullpath: " + pathToImage.fullPath);
    // console.log("nativeURL: " + pathToImage.nativeURL);
    // console.log("filename: " + pathToImage.name);

    pathToImage.getMetadata(
      (metadata: Metadata) => {
        console.log("got image metadata");
        console.log(metadata.size);
      },
      (error) => { console.error(error); }
    );

    // let loader = this.loadingCtrl.create({
    //   content: "Uploading..."
    // });
    // loader.present();
    const fileTransfer: FileTransferObject = this.transfer.create();

    console.log("imageable_id: " + this.appData.issue.issueid);

    // let params = [
    //   {"imageable_id": this.appData.issue.issueid}, 
    //   {"imageable_type": "marker"}
    //   // {"image": ""}
    // ];

    let params = {'imageable_id': this.appData.issue.issueid, 'imageable_type': 'marker'};


    // reqParams.options.headers.append("ContentType", "multipart/form-data");

    let headerPar: {[s: string]: any} = reqParams.options.headers.toJSON();
    console.log("headerPar: " + JSON.stringify(headerPar));

    let options: FileUploadOptions = {
      fileKey: 'image',
      fileName: pathToImage.name,
      httpMethod: "POST",
      chunkedMode: false,
      mimeType: "image/jpg",
      params: params,
      headers: headerPar
    }

    // let host = "http://localhost:3000";
    // let uploadAPIAdress = host + "/api/containers/container1/upload";

    let uploadAPIAdress = UrlProvider.getConnectUrl(reqParams.urlCfg);
    console.log("trying to upload file to: " + uploadAPIAdress);

    fileTransfer.upload(imgToUpload, uploadAPIAdress, options)
      .then((data) => {
        console.log(data + "Successfully uploaded image!");
        this.imageFileName = uploadAPIAdress;
        // loader.dismiss();
      }, (err) => {
        console.error("error on image upload");
        console.log(err);
        // loader.dismiss();
        this.guiUtil.showInfo(err.message);
      });
  }

  // presentToast(msg) {
  //   let toast = this.toastCtrl.create({
  //     message: msg,
  //     duration: 3000,
  //     position: 'bottom'
  //   });

  //   toast.onDidDismiss(() => {
  //     console.log('Dismissed toast');
  //   });

  //   toast.present();
  // }
  //#endregion


  // #region unused
  processBase64(imageData: string) {
    // imageData is a base64 encoded string
    console.log("successfully took picture. add it to the model...");
    let picture = "data:image/jpeg;base64," + imageData;
    // console.log("picture taken: " + picture);
    this.appData.issue.images.push(picture);
    this.changeDetector.detectChanges();
  }


  addFile(result, uri) {
    console.log('Directory exists');
    console.log(result);

    this.file.readAsDataURL(this.file.dataDirectory, uri)
      .then(() => {
        this.appData.issue.images.push(uri);
      })
      .catch((e) => this.onErrorPhotoSelect("addFile: ", e));

  }
  //#endregion


}
