import { Component } from '@angular/core';
import { IonicPage, NavController, NavParams } from 'ionic-angular';
import { FormBuilder, FormGroup, Validators  } from '@angular/forms';
import { AppConfig } from '../../app/app-cfg';
import { LoginPage } from '../login/login';
import { AbstractControl } from '@angular/forms/src/model';
import { ValidationErrors } from '@angular/forms/src/directives/validators';
import { Observable } from 'rxjs/Observable';
import { UserProvider } from '../../providers/user/user';
import { MapPage } from '../map/map';

/**
 * Generated class for the RegisterPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@IonicPage()
@Component({
  selector: 'page-register',
  templateUrl: 'register.html',
})
export class RegisterPage {

  registerForm: FormGroup;

  constructor(public navCtrl: NavController, 
              public navParams: NavParams,
              private formBuilder: FormBuilder,
              private user: UserProvider) {
    
    this.registerForm = this.formBuilder.group({
      email: [
        '', 
          Validators.compose([
          Validators.email,
          Validators.required
          ])
          // ,Validators.composeAsync([this.userExists])
      ],
      password: [
        '', Validators.compose([
          Validators.pattern(AppConfig.PWD_PATTERN),
          Validators.required
        ])
      ],
      confirmPassword: [
        '', Validators.compose([
          Validators.pattern(AppConfig.PWD_PATTERN),
          Validators.required
        ])
      ]
    }, {
      validator: this.MatchPassword // your validation method
    }
  );
  }

  ionViewDidLoad() {
    console.log('ionViewDidLoad RegisterPage');
  }

  onRegister() {
    console.log("TODO: register...");

    if (this.registerForm.valid) {
      var email =  this.registerForm.controls['email'].value;
      var pwd = this.registerForm.controls['password'].value;

      this.user.register(email, pwd)
        .then(() => {
          console.log("login page: login success");
          this.navCtrl.setRoot(LoginPage);
        })
        .catch(() => {
          console.log("login page: login failed");
          this.registerForm.setErrors({'noregister': true});
          // const login = new FormControl("someLogin");
          // login.setErrors({
          //   "notUnique": true
          // });          
          // this.credentialsForm.controls['email'].setErrors({
          //   "emailAlreadyExist": true
          // });
        });
    } 
  }

  private MatchPassword(absControl: AbstractControl) {

    var pwdCtrl = absControl.get('password');
    var confirmCtrl = absControl.get('confirmPassword');

    if(pwdCtrl.value != confirmCtrl.value) {
      console.log("password not equal");
      confirmCtrl.setErrors( {MatchPassword: true} );
    } else {
      console.log("password confirm ok");
      return null;
    }
  }  

  private userExists(control: AbstractControl) : Promise<any> {
    // The userName to test.
    const userName = control.value;
  
    // TODO: replace this observable with an HTTP request
    const existingUsernames = ['kevin@t.at', 'vince@t.at', 'bernard@t.at'];

    return new Promise((resolve, reject) => {
      // If the username is taken, emit an error
      if (existingUsernames.indexOf(userName) !== -1) {
        reject({usernameTaken: true});
      }
      // Username is available, emit null
      else {
        resolve({usernameTaken: false});
      }
    });

    // return Observable.create(observer => {
    //   // If the username is taken, emit an error
    //   if (existingUsernames.indexOf(userName) !== -1) {
    //     observer.next({usernameTaken: true});
    //   }
    //   // Username is available, emit null
    //   else {
    //     observer.next(null);
    //   }
    //   // The observable MUST complete.
    //   observer.complete();
    // });
  }

  goBack() {
    this.navCtrl.push(MapPage);
  }  
  
}
