import { Component, ViewChild } from '@angular/core';
import { IonicPage, NavController, NavParams, Events, AlertController } from 'ionic-angular';
import { MapPage } from '../map/map';
import { AppData } from '../../models/app-data';
import { UserProvider } from '../../providers/user/user';
import { AppConfig } from '../../app/app-cfg';
import { BackendProvider } from '../../providers/backend/backend';
import { UrlModel } from '../../models/url-model';
import { RequestModel } from '../../models/request-model';
import { IssueModel } from '../../models/issue-model';
import { ShowDetailComponent } from '../../components/show-detail/show-detail';
import { IssueManager } from '../map/issue-manager';

/**
 * Generated class for the SummaryPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@IonicPage()
@Component({
  selector: 'page-summary',
  templateUrl: 'summary.html',
})
export class SummaryPage {

  // urgencyLabels = { 0: "no", 1: "Niedrig", 2: "Mittel", 3: "Hoch"};
  @ViewChild(ShowDetailComponent) showDetail: ShowDetailComponent;

  constructor(
    private appData: AppData,
    public navCtrl: NavController, 
    public navParams: NavParams,
    public events: Events,
    private backend: BackendProvider,
    private user: UserProvider,
    private alertCtrl: AlertController,
    private issueManager: IssueManager
  ) {
  
    console.log('Hello Summary Page');
    // this.events.publish(AppConfig.MSGEVT_CHECKSAVE, {});
  }

  ionViewDidLoad() {
    console.log('ionViewDidLoad SummaryPage');
    // let paramData = this.navParams.data;
    // if(paramData) {
    //   let issue: IssueModel = <IssueModel>paramData.data;
    //   this.appData.issue = issue;
    // }
  }
  
  changeLocation() {
    alert("not implemented yet");
  }

  persist() {

    // this.appData.getDeviceId()
    // .then( (uuid: any) => {
    // // todo: use it later for unauthenticated users
    // console.log("device id: " + uuid);
    // this.events.publish(AppConfig.MSGEVT_NAV, { direction: AppConfig.NAV_SAVE });
    // });

    // get user profile
    var urlCfg: UrlModel = { 
      "path": "/backendapi/",
      "proxyUrl": AppConfig.BACKEND_API_DOMAIN,
      "resource": "/api/v1/gehfahrlos/issues/" + this.appData.issue.issueid
    }

    let bodyStr = JSON.stringify(this.appData.issue);
    console.log(bodyStr);
    let reqParams: RequestModel = {urlCfg: urlCfg, body: bodyStr};
    
    this.backend.authPostRequest(reqParams)
      .then((response)=> {
        // var address = response.json().results[0].formatted_address;
        // var email = response.json().email;
        console.log("successfully saved issue data!");
        this.issueManager.transformMarkerSaved();
        this.appData.status = AppData.STATUS_AUTH;
        this.confirmSave();
        this.events.publish(AppConfig.MSGEVT_NAV, { direction: AppConfig.NAV_SAVE });
    });
    
  }

  switchTab(targetIdx: number) {
    this.events.publish(AppConfig.MSGEVT_TABSELECT, { target: targetIdx });
  }


  confirmSave() {

    let alert = this.alertCtrl.create({
      title: "Gefahrenstelle markiert!",
      subTitle: "Danke für die Übermittlung. Ihr Eintrag wurde erfolgreich gespeichert.",
      buttons: ['OK']
    });
  alert.present();
}
 

}