import { Injectable } from '@angular/core';
import { ResponseOptions, Http, RequestOptionsArgs,RequestOptions, Headers, URLSearchParams } from '@angular/http';
import 'rxjs/add/operator/map';
import { UrlModel } from '../../models/url-model';
import { UserProvider } from '../user/user';
import { retry } from 'rxjs/operator/retry';
import { AppConfig } from '../../app/app-cfg';
import { IssueModel } from '../../models/issue-model';
import { resolveDefinition } from '@angular/core/src/view/util';
import { Observable } from 'rxjs/Observable';
import { } from '@types/googlemaps';
import { UrlProvider } from '../url/url';
import { RequestModel } from '../../models/request-model';
import { Header } from 'ionic-angular';

declare var google;
// import { Response } from '@angular/http/src/static_response';
// import { ResponseOptionsArgs } from '@angular/http/src/interfaces';
/*
  Generated class for the BackendProvider provider.

  See https://angular.io/docs/ts/latest/guide/dependency-injection.html
  for more info on providers and Angular 2 DI.
*/
@Injectable()
export class BackendProvider {


  private reqCallback: (reqParams: RequestModel) => Promise<any>;


  // public issues: IssueModel[];
  // issuesMap: Map<number, IssueModel>;


  constructor(public http: Http,
              private user: UserProvider) {
    console.log('Hello BackendProvider');
    // this.issues = new Array<IssueModel>();
    // this.issuesMap = new Map<number, IssueModel>();
  }


  public simpleGetRequest(reqParams: RequestModel): Promise<any> {
    this.reqCallback = this.execGet;
    return this.execGet(reqParams);
  }

  public simplePostRequest(reqParams: RequestModel): Promise<any> {
    this.reqCallback = this.execPost;
    return this.execPost(reqParams);
  }

  public authGetRequest(reqParams: RequestModel): Promise<any> {
    this.reqCallback = this.execGet;
    return this.makeAuthRequest(reqParams);
  }

  public authPostRequest(reqParams: RequestModel): Promise<any> {
    this.reqCallback = this.execPost;
    return this.makeAuthRequest(reqParams);
  }


  public makeAuthRequest(reqParams: RequestModel): Promise<any> {
    
    return new Promise( (resolve, reject) => {
      // provide security credentials through header
      this.user.authenticateRequest(reqParams)
      .then((paramsAuth: RequestModel) => {
        this.reqCallback(paramsAuth)
        // this.getRequest(urlCfg, options)
          .then((data) => { resolve(data); })
          .catch( (err) => { reject(err); })
      });
    });
  }

  public getAuthRequestHeader(reqParams: RequestModel): Promise<any> {
    return this.user.authenticateRequest(reqParams);
  }
  
  private execGet(reqParams: RequestModel): Promise<any> {
    
    return new Promise( (resolve, reject) => {

      let connectUrl = UrlProvider.getConnectUrl(reqParams.urlCfg);
      
      console.log("request the API url: " + connectUrl);

      this.http.get(connectUrl, reqParams.options)
      // .map(this.mapResponse)// res.json();)
      .subscribe(
        (response) => {
          console.log("got response from server");
          // console.log(data);
          // this.handleAPIResponse(data);
          resolve(response);
        }, 
        (err) => { this.handleError(err);}
      );
    });
  }

  private execPost(reqParams: RequestModel): Promise<any> {
    
    return new Promise( (resolve, reject) => {

      let connectUrl = UrlProvider.getConnectUrl(reqParams.urlCfg);
      
      console.log("post request url: " + connectUrl);

      this.http.post(connectUrl, reqParams.body, reqParams.options)
      .subscribe(
        (response) => {
          console.log("got response from server");
          resolve(response);
        }, 
        (err) => { this.handleError(err);}
      );
    });
  }



  private handleError(error: Response): Promise<any> {
    
    console.error('error when REST API call', error); 
    // error.json().then((data) => { console.error(data) });

    let errobj = error.json();
    // if(errobj && errobj.error_message) {
    //   console.error(errobj.error_message);
    // }

    return Promise.reject(error.statusText || error);
  }


  public testThis() {

    var urlCfg = {
      // "path": "/baseurl/",
      // "proxyUrl": "https://1pbwvz80qj.execute-api.eu-west-1.amazonaws.com/api/followmebackend/",
      // "resource": "webresources/ro.siveco.followme.followmebackend.patients"

      "path": "/guestapi",
      "proxyUrl": "https://xk9nynv88f.execute-api.eu-central-1.amazonaws.com/test/getall",
      "resource": ""
      
      // "path": "/baseurltest/",
      // "proxyUrl": "https://xk9nynv88f.execute-api.eu-central-1.amazonaws.com/test/"
    };
      
    let reqParams: RequestModel = {urlCfg: urlCfg};
    this.simpleGetRequest(reqParams)
        .then((response)=> {
      console.log("test: got response from guestapi!");
    });
  }

  private getTestData() {

    var issStr = '[\
      {"deviceid": null, "issueid":8168,"importance":null,"description":null,"location":{"address":null,"lat":48.17875119005036,"lng":16.374032137091355},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":2,   "importance":null,"description":null,"location":{"address":"G\u00f6tzgasse 3, 1100 Wien, Austria","lat":48.1788416,"lng":16.373018199999933},"images":[{"id":1,"url":"\/storage\/images\/marker\/13843\/1\/basic.png"},{"id":2,"url":"\/storage\/images\/marker\/13843\/2\/basic.png"}],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":5396,"importance":null,"description":null,"location":{"address":null,"lat":48.1789,"lng":16.3700816},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":5399,"importance":null,"description":null,"location":{"address":null,"lat":48.1791136,"lng":16.3696852},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":5330,"importance":null,"description":null,"location":{"address":null,"lat":48.1794483,"lng":16.3770761},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":5329,"importance":null,"description":null,"location":{"address":null,"lat":48.1794576,"lng":16.3770627},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":6526,"importance":null,"description":null,"location":{"address":null,"lat":48.1808529,"lng":16.3735194},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":7321,"importance":null,"description":null,"location":{"address":null,"lat":48.1811586,"lng":16.3783054},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":7318,"importance":null,"description":null,"location":{"address":null,"lat":48.1813864,"lng":16.3775349},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":1897,"importance":null,"description":null,"location":{"address":null,"lat":48.1821038,"lng":16.3779896},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":1898,"importance":null,"description":null,"location":{"address":null,"lat":48.1821147,"lng":16.3779792},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":7918,"importance":null,"description":null,"location":{"address":null,"lat":48.18399416692286,"lng":16.372067153332818},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":7923,"importance":null,"description":null,"location":{"address":null,"lat":48.184647166021314,"lng":16.368694171416305},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]}\
      ]\
    ';
    
    return JSON.parse(issStr);
  }
  
  private getTestData2(): string {

    var issueArr = '{"data": {\
      "deviceid": null,\
      "full":[\
      {"deviceid": null, "issueid":8168,"importance":null,"description":null,"location":{"address":null,"lat":48.17875119005036,"lng":16.374032137091355},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":2,   "importance":null,"description":null,"location":{"address":"G\u00f6tzgasse 3, 1100 Wien, Austria","lat":48.1788416,"lng":16.373018199999933},"images":[{"id":1,"url":"\/storage\/images\/marker\/13843\/1\/basic.png"},{"id":2,"url":"\/storage\/images\/marker\/13843\/2\/basic.png"}],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":5396,"importance":null,"description":null,"location":{"address":null,"lat":48.1789,"lng":16.3700816},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":5399,"importance":null,"description":null,"location":{"address":null,"lat":48.1791136,"lng":16.3696852},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":5330,"importance":null,"description":null,"location":{"address":null,"lat":48.1794483,"lng":16.3770761},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":5329,"importance":null,"description":null,"location":{"address":null,"lat":48.1794576,"lng":16.3770627},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":6526,"importance":null,"description":null,"location":{"address":null,"lat":48.1808529,"lng":16.3735194},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":7321,"importance":null,"description":null,"location":{"address":null,"lat":48.1811586,"lng":16.3783054},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":7318,"importance":null,"description":null,"location":{"address":null,"lat":48.1813864,"lng":16.3775349},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":1897,"importance":null,"description":null,"location":{"address":null,"lat":48.1821038,"lng":16.3779896},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":1898,"importance":null,"description":null,"location":{"address":null,"lat":48.1821147,"lng":16.3779792},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":7918,"importance":null,"description":null,"location":{"address":null,"lat":48.18399416692286,"lng":16.372067153332818},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]},\
      {"deviceid": null, "issueid":7923,"importance":null,"description":null,"location":{"address":null,"lat":48.184647166021314,"lng":16.368694171416305},"images":[],"targetList":[{"participant":{"label":"Fu\u00dfg\u00e4nger","value":true},"hazardList":[{"label":"enger Gehweg","value":true},{"label":"fehlender Gehweg","value":true},{"label":"Ampelanlage","value":true},{"label":"Sch\u00e4den \/ Stolpergefahr","value":true},{"label":"Kanaldeckel \/ Gitter","value":true},{"label":"Beschilderung","value":true},{"label":"Verunreinigung","value":true},{"label":"Hindernisse","value":true}]},{"participant":{"label":"\u00d6ffentliche Verkehrsmittel","value":true},"hazardList":[{"label":"Beschilderung","value":true},{"label":"Gestaltung Haltestelle","value":true},{"label":"Vandalismus","value":true},{"label":"kein barrierefreier Einstieg","value":true}]},{"participant":{"label":"Radfahrer","value":true},"hazardList":[{"label":"Stra\u00dfenzustand","value":true},{"label":"enge Fahrspur","value":true},{"label":"rutschige Oberfl\u00e4che","value":true},{"label":"Abstellm\u00f6glichkeiten","value":true},{"label":"L\u00fcckenschluss","value":true},{"label":"Ampelschaltung","value":true},{"label":"Beschilderung","value":true}]},{"participant":{"label":"PKW","value":true},"hazardList":[{"label":"Hoher Randstein","value":true},{"label":"Un\u00fcbersichtliche Kreuzung","value":true},{"label":"Beschilderung","value":true},{"label":"Baustelle","value":true}]}]}\
      ]\
      }\
    ';

    return issueArr;
    // var optionArgs = {
    //   body: JSON.stringify(issueArr),
    //   status: 200
    // };

    // let resp = new Response(new ResponseOptions(optionArgs));
    // return Observable.of(resp);
    
  }
  
}
