import { Injectable } from '@angular/core';
import { Http } from '@angular/http';
import 'rxjs/add/operator/map';
// import {} from '@types/googlemaps';

/*
  Generated class for the MapService provider.

  See https://angular.io/docs/ts/latest/guide/dependency-injection.html
  for more info on providers and Angular 2 DI.
*/
@Injectable()
export class MapService {

  public static MAPSTYLE_1: number = 1;
  public static MAPSTYLE_2: number = 2;
  public static MAPSTYLE_3: number = 3;
  public static MAPSTYLE_4: number = 4;
  
  public static MARKER_1: string =  'data:image/svg+xml;utf-8, \
    <svg fill="#c23d11" height="24" viewBox="0 0 24 24" width="24" xmlns="http://www.w3.org/2000/svg"> \
    <path d="M 12,22 C 8.14,22 5,18.86 5,15 5,9.75 12,2 12,2 c 0,0 7,7.75 7,13 0,3.86 -3.14,7 -7,7 z M 10.44,12 9,12 9,13.44 12.35,16.78 13.78,15.35 10.44,12 Z m 4.45,4.45 -0.7,-0.7 -1.44,1.44 0.7,0.7 c 0.15,0.15 0.39,0.15 0.54,0 l 0.9,-0.9 c 0.15,-0.15 0.15,-0.39 0,-0.54 z"/> \
    <path d="M0 0h24v24H0z" fill="none"/> \
    </svg> \
  ';

  public static MARKER_2: string =  'data:image/svg+xml;utf-8, \
    <svg width="110" height="110" version="1.1" xmlns="http://www.w3.org/2000/svg"> \
    <polyline points="0 5, 100 5, 100 30, 52.5 100, 5 30, 5 5" stroke="orange" fill="#555555" stroke-width="10" /> \
    </svg> \
  ';

  private mapStyleDefault: any = MapService.MAPSTYLE_4;

  constructor() {
    console.log('Hello MapService Provider');
  }

  public setDefaultStyle(mapStyle: any) {
    this.mapStyleDefault = mapStyle;
  }

  getMarker(myLatLng: google.maps.LatLng, 
            markerIcon: string|google.maps.Icon|google.maps.Symbol,
            markerCaption: string
            ): google.maps.Marker {
    
    console.log("create marker: " + myLatLng.toString());

    var marker = new google.maps.Marker({
        position: myLatLng,
        icon: markerIcon,
        draggable: false,
        title: markerCaption,
    });


    return marker;
  }
  
  getIcon(thumbnailurl: string) {

    // return {
    //   path: 'M 125,5 155,90 245,90 175,145 200,230 125,180 50,230 75,145 5,90 95,90 z',
    //   fillColor: 'yellow',
    //   fillOpacity: 0.8,
    //   scale: 1,
    //   strokeColor: 'gold',
    //   strokeWeight: 14
    // };

    // var icon = {
    //   path: "M 675.41391, 2.2069197 C 631.36583,0.70665572 92.510751,6.7088973 67.550214,6.7088973 42.589674,6.7088973 8.8192163,38.210983 8.8192163,83.21853 L 5.889516,753.84422 375.87672,1044.8921 745.88661,755.32939 c 0,0 -4.41393,-654.10438 -5.8822,-688.610193 C 738.53615,32.213377 719.4621,3.7071828 675.41391,2.2069197 Z M 372.94691,70.849879 A 276.40212,379.93919 0 0 1 649.34149,450.77592 276.40212,379.93919 0 0 1 372.94691,830.72525 276.40212,379.93919 0 0 1 96.552339,450.77592 276.40212,379.93919 0 0 1 372.94691,70.849879 Z",
    //   fillColor: '#5aa9d346',
    //   fillOpacity: .6,
    //   thumbnail: person.picture.thumbnail,
    //   anchor: new google.maps.Point(375,1000),
    //   strokeWeight: 0,
    //   scale: 0.05
    // };

    // console.log("thumbnail url:");
    // console.log(thumbnailurl);

    var image1: google.maps.Icon = {
      url: thumbnailurl,
      origin: new google.maps.Point(0, 0),
      size: new google.maps.Size(60, 60),
      scaledSize: new google.maps.Size(60, 60),
      anchor: new google.maps.Point(30, 60)
    }        

    // var image2: google.maps.Symbol = {
    //   path: "M12 2C8.14 2 5 5.14 5 9c0 5.25 7 13 7 13s7-7.75 7-13c0-3.86-3.14-7-7-7zm-1.56 10H9v-1.44l3.35-3.34 1.43 1.43L10.44 12zm4.45-4.45l-.7.7-1.44-1.44.7-.7c.15-.15.39-.15.54 0l.9.9c.15.15.15.39 0 .54z",
    //   anchor: new google.maps.Point(12, 24),
    //   fillColor: "#000000",
    //   scale: 3,
    //   rotation: 180
    // }        
      
    return image1;
  }

  getIconPixel(thumbnailurl) {
    var image = {
    url: thumbnailurl,
      // This marker is 20 pixels wide by 32 pixels high.
      size: new google.maps.Size(40, 60),
      scale: 0.3
      // The origin for this image is (0, 0).
      // origin: new google.maps.Point(0, 0),
      // The anchor for this image is the base of the flagpole at (0, 32).
      // anchor: new google.maps.Point(0, 32)
    };
  }

  getMapStyleDefs(styleType: number) {
    
    if(styleType===null) {
      styleType = this.mapStyleDefault;
    }

    switch(styleType) {
      case MapService.MAPSTYLE_1: {
        return this.getStyleDef1();
      }
      case MapService.MAPSTYLE_2: {
        return this.getStyleDef2();
      }
      case MapService.MAPSTYLE_3: {
        return this.getStyleDef3();
      }
    }

    return this.getStyleDef4();
  }

  private getStyleDef1() {

    return new google.maps.StyledMapType(
      [
        {
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#242f3e"
            }
          ]
        },
        {
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#746855"
            }
          ]
        },
        {
          "elementType": "labels.text.stroke",
          "stylers": [
            {
              "color": "#242f3e"
            }
          ]
        },
        {
          "featureType": "administrative.locality",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#d59563"
            }
          ]
        },
        {
          "featureType": "poi",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#d59563"
            }
          ]
        },
        {
          "featureType": "poi.park",
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#263c3f"
            }
          ]
        },
        {
          "featureType": "poi.park",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#6b9a76"
            }
          ]
        },
        {
          "featureType": "road",
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#38414e"
            }
          ]
        },
        {
          "featureType": "road",
          "elementType": "geometry.stroke",
          "stylers": [
            {
              "color": "#212a37"
            }
          ]
        },
        {
          "featureType": "road",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#9ca5b3"
            }
          ]
        },
        {
          "featureType": "road.arterial",
          "elementType": "geometry.fill",
          "stylers": [
            {
              "color": "#ffeb3b"
            }
          ]
        },
        {
          "featureType": "road.highway",
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#746855"
            }
          ]
        },
        {
          "featureType": "road.highway",
          "elementType": "geometry.fill",
          "stylers": [
            {
              "color": "#f06e28"
            }
          ]
        },
        {
          "featureType": "road.highway",
          "elementType": "geometry.stroke",
          "stylers": [
            {
              "color": "#1f2835"
            }
          ]
        },
        {
          "featureType": "road.highway",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#f3d19c"
            }
          ]
        },
        {
          "featureType": "road.local",
          "elementType": "geometry.fill",
          "stylers": [
            {
              "color": "#b9b9ff"
            }
          ]
        },
        {
          "featureType": "transit",
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#2f3948"
            }
          ]
        },
        {
          "featureType": "transit.station",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#d59563"
            }
          ]
        },
        {
          "featureType": "water",
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#17263c"
            }
          ]
        },
        {
          "featureType": "water",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#515c6d"
            }
          ]
        },
        {
          "featureType": "water",
          "elementType": "labels.text.stroke",
          "stylers": [
            {
              "color": "#17263c"
            }
          ]
        }
      ]
    );
  }

  private getStyleDef2() {

    return new google.maps.StyledMapType(
      [
        {
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#242f3e"
            }
          ]
        },
        {
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#746855"
            }
          ]
        },
        {
          "elementType": "labels.text.stroke",
          "stylers": [
            {
              "color": "#242f3e"
            }
          ]
        },
        {
          "featureType": "administrative.locality",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#d59563"
            }
          ]
        },
        {
          "featureType": "poi",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#d59563"
            }
          ]
        },
        {
          "featureType": "poi.park",
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#263c3f"
            }
          ]
        },
        {
          "featureType": "poi.park",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#6b9a76"
            }
          ]
        },
        {
          "featureType": "road",
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#38414e"
            }
          ]
        },
        {
          "featureType": "road",
          "elementType": "geometry.stroke",
          "stylers": [
            {
              "color": "#212a37"
            }
          ]
        },
        {
          "featureType": "road",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#9ca5b3"
            }
          ]
        },
        {
          "featureType": "road.arterial",
          "elementType": "geometry.fill",
          "stylers": [
            {
              "color": "#f17532"
            },
            {
              "weight": 1
            }
          ]
        },
        {
          "featureType": "road.arterial",
          "elementType": "geometry.stroke",
          "stylers": [
            {
              "color": "#ffeb3b"
            }
          ]
        },
        {
          "featureType": "road.highway",
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#746855"
            }
          ]
        },
        {
          "featureType": "road.highway",
          "elementType": "geometry.fill",
          "stylers": [
            {
              "color": "#f8b692"
            }
          ]
        },
        {
          "featureType": "road.highway",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#f3d19c"
            }
          ]
        },
        {
          "featureType": "road.local",
          "elementType": "geometry.fill",
          "stylers": [
            {
              "color": "#b9b9ff"
            }
          ]
        },
        {
          "featureType": "transit",
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#2f3948"
            }
          ]
        },
        {
          "featureType": "transit.station",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#d59563"
            }
          ]
        },
        {
          "featureType": "water",
          "elementType": "geometry",
          "stylers": [
            {
              "color": "#17263c"
            }
          ]
        },
        {
          "featureType": "water",
          "elementType": "labels.text.fill",
          "stylers": [
            {
              "color": "#515c6d"
            }
          ]
        },
        {
          "featureType": "water",
          "elementType": "labels.text.stroke",
          "stylers": [
            {
              "color": "#17263c"
            }
          ]
        }
      ]
    );
  }

  private getStyleDef3() {

    return new google.maps.StyledMapType(
        [
          {
            "elementType": "geometry",
            "stylers": [
              {
                "color": "#f5f5f5"
              }
            ]
          },
          {
            "elementType": "labels.icon",
            "stylers": [
              {
                "visibility": "off"
              }
            ]
          },
          {
            "elementType": "labels.text.fill",
            "stylers": [
              {
                "color": "#616161"
              }
            ]
          },
          {
            "elementType": "labels.text.stroke",
            "stylers": [
              {
                "color": "#f5f5f5"
              }
            ]
          },
          {
            "featureType": "administrative.land_parcel",
            "elementType": "labels.text.fill",
            "stylers": [
              {
                "color": "#bdbdbd"
              }
            ]
          },
          {
            "featureType": "poi",
            "elementType": "geometry",
            "stylers": [
              {
                "color": "#eeeeee"
              }
            ]
          },
          {
            "featureType": "poi",
            "elementType": "labels.text.fill",
            "stylers": [
              {
                "color": "#757575"
              }
            ]
          },
          {
            "featureType": "poi.medical",
            "elementType": "geometry.fill",
            "stylers": [
              {
                "color": "#ed6047"
              }
            ]
          },
          {
            "featureType": "poi.park",
            "elementType": "geometry",
            "stylers": [
              {
                "color": "#e5e5e5"
              }
            ]
          },
          {
            "featureType": "poi.park",
            "elementType": "labels.text.fill",
            "stylers": [
              {
                "color": "#9e9e9e"
              }
            ]
          },
          {
            "featureType": "road",
            "elementType": "geometry",
            "stylers": [
              {
                "color": "#ffffff"
              }
            ]
          },
          {
            "featureType": "road.arterial",
            "elementType": "labels.text.fill",
            "stylers": [
              {
                "color": "#757575"
              }
            ]
          },
          {
            "featureType": "road.highway",
            "elementType": "geometry",
            "stylers": [
              {
                "color": "#dadada"
              }
            ]
          },
          {
            "featureType": "road.highway",
            "elementType": "labels.text.fill",
            "stylers": [
              {
                "color": "#616161"
              }
            ]
          },
          {
            "featureType": "road.local",
            "elementType": "labels.text.fill",
            "stylers": [
              {
                "color": "#9e9e9e"
              }
            ]
          },
          {
            "featureType": "transit.line",
            "elementType": "geometry",
            "stylers": [
              {
                "color": "#e5e5e5"
              }
            ]
          },
          {
            "featureType": "transit.station",
            "elementType": "geometry",
            "stylers": [
              {
                "color": "#eeeeee"
              }
            ]
          },
          {
            "featureType": "water",
            "elementType": "geometry",
            "stylers": [
              {
                "color": "#c9c9c9"
              }
            ]
          },
          {
            "featureType": "water",
            "elementType": "geometry.fill",
            "stylers": [
              {
                "color": "#c7e8fc"
              },
              {
                "saturation": -5
              },
              {
                "weight": 1
              }
            ]
          },
          {
            "featureType": "water",
            "elementType": "labels.text.fill",
            "stylers": [
              {
                "color": "#9e9e9e"
              }
            ]
          }
        ]      
    );
  }

  private getStyleDef4() {
    return new google.maps.StyledMapType(
      [
        {
          "featureType": "landscape.natural.terrain",
          "stylers": [
            {
              "color": "#9cc53f"
            }
          ]
        },
        {
          "featureType": "poi.park",
          "stylers": [
            {
              "color": "#9cc53f"
            }
          ]
        },
        {
          "featureType": "water",
          "stylers": [
            {
              "color": "#44a2d5"
            }
          ]
        }
      ]
    );
          
  }
}
