import { Injectable } from '@angular/core';
import { Http, RequestOptionsArgs, RequestOptions, Headers } from '@angular/http';
import {Observable, Subject, BehaviorSubject} from 'rxjs/Rx';
import 'rxjs/add/operator/map';
import { UrlModel } from '../../models/url-model';
import { AppConfig } from '../../app/app-cfg';
import { UrlProvider } from '../url/url';
import { TokenModel } from '../../models/token-model';
import { RequestModel } from '../../models/request-model';
import { UserModel } from '../../models/user-model';

/*
  Generated class for the UserProvider provider.

  See https://angular.io/docs/ts/latest/guide/dependency-injection.html
  for more info on providers and Angular 2 DI.
*/
@Injectable()
export class UserProvider {

  private static appCred = { 
    'client_id': "3",
    'client_secret': "3vaV7bUTKet03M6KqpP7MbZ2uO0Uk9SoYUCX64NJ",
  };

  private isAuth$: BehaviorSubject<TokenModel> = new BehaviorSubject<TokenModel>({access_token: null, expires_in: null, refresh_token: null});

  private APP_PREFIX: string = "com$gehfahrlos$";
  private CURUSER_PREFIX: string = "user";
  private TOKEN_ACCESS_PREFIX: string = "access";
  private TOKEN_EXPIRE_PREFIX: string = "expire";
  private TOKEN_REFRESH_PREFIX: string = "refresh";

  private curUserName: string;
  private profile: UserModel;

  constructor(public http: Http) {

    console.log('Hello UserProvider Provider');

    this.readUser();
  }

  get Profile() {
    return this.profile;
  }


  register(email, password): Promise<any> {

    return new Promise((resolve, reject) => {

      console.log("TODO: Try to register...");

      // TODO: http.get...
      resolve("successfully registered");
      // reject("couldn't register!");
    });
  }

  private getLoginBody(username, password) {
    return {
      'client_id': UserProvider.appCred.client_id,
      'client_secret': UserProvider.appCred.client_secret,
      'grant_type': "password",
      'username': username,
      'password': password
    };   
  }

  login(username, password): Promise<TokenModel> {

    return new Promise((resolve, reject) => {

      console.log("Try to login...");

      // TODO: http.get...
      var urlCfg: UrlModel = { 
        // "path": "/geocode",
        "path": "/backendapi/",
        "proxyUrl": AppConfig.BACKEND_API_DOMAIN,
        "resource": "oauth/token"
      }

      const body = this.getLoginBody(username, password);
      let connectUrl = UrlProvider.getConnectUrl(urlCfg);
      
      let optionParams = UrlProvider.createOptions();
      console.log(JSON.stringify(optionParams));

      this.http.post(connectUrl, body, optionParams)
      .subscribe(
        (response) => {
          // "token_type": "Bearer",
          // "expires_in": 863999,
          // "access_token":
          // refresh_token
          console.log("successfully logged in!");
          let token: TokenModel = response.json();
          // save tmp token ==> current logged in user
          this.saveUser(username);
          this.saveToken(token);

          this.requestUserProfile(token.access_token);
          this.isAuth$.next(token);
          
          // resObj.access_token;
          resolve(token);
        }, 
        (err) => { 
          this.logout();
          console.log(err);
          reject();
        }
      );

    });
  }

  requestUserProfile(token: string) {

    console.log("trying to get user profile...");

    var urlCfg: UrlModel = { 
      "path": "/backendapi/",
      "proxyUrl": AppConfig.BACKEND_API_DOMAIN,
      "resource": "api/v1/gehfahrlos/users/current"
    }

    // this.backend.callAPI(urlCfg, searchParam)
    let reqParams: RequestModel = {urlCfg: urlCfg};

    let connectUrl = UrlProvider.getConnectUrl(urlCfg);

    // let token = this.getToken();
    reqParams = this.addTokenToRequest(reqParams, token);
    this.http.get(connectUrl, reqParams.options)
    // .map(this.mapResponse)// res.json();)
    .subscribe(
      (response) => {
        console.log("got user profile");
        this.profile = <UserModel>response.json().data;
        // console.log(data);
        // this.handleAPIResponse(data);
      }, 
      (err) => { 
        console.error("error when trying to get user profile!");
        console.error(err);
      }
    );
  }
  
 
  logout(): void {
    // clear token remove user from local storage to log user out
    // localStorage.removeItem('currentUser');


    this.removeToken(this.TOKEN_ACCESS_PREFIX);
    this.removeToken(this.TOKEN_EXPIRE_PREFIX);
    this.removeToken(this.TOKEN_REFRESH_PREFIX);

    let userKey = this.getUserKey();
    window.localStorage.removeItem(userKey);

    this.curUserName = null;
    this.isAuth$.next(null);
    
  }

  public getAuthSubject() {

    return this.isAuth$;
  }

  // public isAuthenticatedSync(): boolean {
    
  //   let token = this.getToken();
  //   if(token && token.access_token) {
  //     return true;
  //   }
  //   return false;
  // }

  public isAuthenticated(): Promise<TokenModel> {
 
    return new Promise((resolve, reject) => {

      this.getToken()
      .then( (token) => {
        resolve(token);
      })
      .catch( (err) => {
        console.log("not authenticated. logout...");
        this.logout();
      });
    });
  }

  private removeToken(prefix: string) {
    let key = this.getTokenKey(prefix);
    window.localStorage.removeItem(key);
  }

  private getEmptyToken(): TokenModel {
    return { access_token: null, expires_in: null, refresh_token: null};
  }

  private getToken(): Promise<TokenModel> {

    let accesskey = this.getTokenKey(this.TOKEN_ACCESS_PREFIX);
    let expirekey = this.getTokenKey(this.TOKEN_EXPIRE_PREFIX);
    let refreshkey = this.getTokenKey(this.TOKEN_REFRESH_PREFIX);

    var token: TokenModel = this.getEmptyToken();
    token.access_token = window.localStorage.getItem(accesskey);
    token.expires_in = parseInt(window.localStorage.getItem(expirekey));
    token.refresh_token = window.localStorage.getItem(refreshkey);

    if(!token.access_token) {
      return new Promise((resolve, reject) => { reject(token);});
    }

    let millisNow = Date.now();
    if(millisNow < token.expires_in) {
      // access token is valid
      console.log("now: " + new Date(millisNow).toISOString());
      console.log("expire: " + new Date(token.expires_in).toISOString());
      console.log("accesss token is valid.");
      return new Promise((resolve) => { resolve(token); });
    } 

    // return new Promise((resolve, reject) => { reject(token);});
    
    console.warn("token expires soon, try to request new one...");
    return this.refreshToken(token.refresh_token);
  }

  refreshToken(refreshToken: string): Promise<TokenModel> {

    return new Promise((resolve, reject) => {

      console.log("Try to request refresh token...");

      var urlCfg: UrlModel = { 
        "path": "/backendapi/",
        "proxyUrl": AppConfig.BACKEND_API_DOMAIN,
        "resource": "oauth/token"
      }

      // refreshToken = "test-fake-refresh-token";

      const body = {
        'client_id': UserProvider.appCred.client_id,
        'client_secret': UserProvider.appCred.client_secret,
        'grant_type': "refresh_token",
        'refresh_token': refreshToken
      };   

      let connectUrl = UrlProvider.getConnectUrl(urlCfg);
      let optionParams = UrlProvider.createOptions();
      console.log(JSON.stringify(optionParams));

      this.http.post(connectUrl, body, optionParams)
      .subscribe(
        (response) => {
          console.log("token refreshed successfully!");
          let token: TokenModel = response.json();
          this.saveToken(token);
          this.isAuth$.next(token);
          resolve(token);
        }, 
        (err) => { 
          this.logout();
          console.log(err);
          reject(this.getEmptyToken());
        }
      );

    });
  }

  private saveUser(username: string) {

    if(!username) {
      console.warn("cannot save user");
      return;
    }

    this.curUserName = username;
    let userkey = this.getUserKey();
    window.localStorage.setItem(userkey, this.curUserName);
  }
  
  private saveToken(token: TokenModel) {

    console.log("try to save token...");
    let accesskey = this.getTokenKey(this.TOKEN_ACCESS_PREFIX);
    let refreshkey = this.getTokenKey(this.TOKEN_REFRESH_PREFIX);
        
    window.localStorage.setItem(accesskey, token.access_token);
    window.localStorage.setItem(refreshkey, token.refresh_token);
    // window.localStorage.setItem(expirekey, token.expires_in);

    this.calcAndSaveExpire(token);
  }

  private calcAndSaveExpire(token: TokenModel) {

    let expiresIn: number = token.expires_in;
    // let expiresIn: number = 60;
    let expireMinus: number = Math.floor(expiresIn / 10);
    let expirediv = expiresIn - expireMinus;
    
    let millisNow = Date.now();
    let nowDate = new Date(millisNow);
    console.log("datenow: " + nowDate.toISOString());
    console.log("add sec: " + expirediv);
    nowDate.setSeconds(expirediv);
    console.log("expire date: " + nowDate.toISOString());

    let expireMillis = nowDate.getTime();
    console.log("expire millis: " + expireMillis);

    let expirekey = this.getTokenKey(this.TOKEN_EXPIRE_PREFIX);
    window.localStorage.setItem(expirekey, expireMillis.toString());
    
  }

  private getTokenKey(prefix: string) {

    let key = this. APP_PREFIX + prefix;

    if(this.curUserName) {
      return key + this.curUserName;
    }

    return key;
  }

  private getUserKey() {
    return this.APP_PREFIX + this.CURUSER_PREFIX;
  }
  
  private readUser() {
    console.log("try to read user...");
    let userkey = this.getUserKey();
    this.curUserName = window.localStorage.getItem(userkey);
  }


  authenticateRequest(reqParams: RequestModel): Promise<RequestModel> {

    return new Promise((resolve, reject) => {
      
      this.isAuthenticated().then((token: TokenModel) => {

        console.log('you are authenticated! Add token to request...');

        reqParams = this.addTokenToRequest(reqParams, token.access_token);

        resolve(reqParams);

      }).catch((reason) => {
        console.info('you are not authenticated..'); 
        // reject(reason);
        resolve(null);
      });

    });
  }

  addTokenToRequest(reqParams: RequestModel, accesstoken: string) {
    if(!reqParams.options) {
      reqParams.options = UrlProvider.createOptions();
    }
    // let token: TokenModel = this.getToken();
    if(!reqParams.options.headers) {
      reqParams.options.headers = UrlProvider.createHeader();
    }
    reqParams.options.headers.append("Authorization", "Bearer " + accesstoken);
    return reqParams;
  }

}